# coding: utf8
from __future__ import unicode_literals

import plac
import math
from tqdm import tqdm
import numpy
from ast import literal_eval
from pathlib import Path
from preshed.counter import PreshCounter
import tarfile
import gzip
import zipfile
import srsly
import warnings
from wasabi import msg

from ..vectors import Vectors
from ..errors import Errors, Warnings
from ..util import ensure_path, get_lang_class, load_model, OOV_RANK
from ..lookups import Lookups


try:
    import ftfy
except ImportError:
    ftfy = None


DEFAULT_OOV_PROB = -20


@plac.annotations(
    lang=("Model language", "positional", None, str),
    output_dir=("Model output directory", "positional", None, Path),
    freqs_loc=("Location of words frequencies file", "option", "f", Path),
    jsonl_loc=("Location of JSONL-formatted attributes file", "option", "j", Path),
    clusters_loc=("Optional location of brown clusters data", "option", "c", str),
    vectors_loc=("Optional vectors file in Word2Vec format", "option", "v", str),
    truncate_vectors=(
        "Optional number of vectors to truncate to when reading in vectors file",
        "option",
        "t",
        int,
    ),
    prune_vectors=("Optional number of vectors to prune to", "option", "V", int),
    vectors_name=(
        "Optional name for the word vectors, e.g. en_core_web_lg.vectors",
        "option",
        "vn",
        str,
    ),
    model_name=("Optional name for the model meta", "option", "mn", str),
    omit_extra_lookups=("Don't include extra lookups in model", "flag", "OEL", bool),
    base_model=("Base model (for languages with custom tokenizers)", "option", "b", str),
)
def init_model(
    lang,
    output_dir,
    freqs_loc=None,
    clusters_loc=None,
    jsonl_loc=None,
    vectors_loc=None,
    truncate_vectors=0,
    prune_vectors=-1,
    vectors_name=None,
    model_name=None,
    omit_extra_lookups=False,
    base_model=None,
):
    """
    Create a new model from raw data, like word frequencies, Brown clusters
    and word vectors. If vectors are provided in Word2Vec format, they can
    be either a .txt or zipped as a .zip or .tar.gz.
    """
    if jsonl_loc is not None:
        if freqs_loc is not None or clusters_loc is not None:
            settings = ["-j"]
            if freqs_loc:
                settings.append("-f")
            if clusters_loc:
                settings.append("-c")
            msg.warn(
                "Incompatible arguments",
                "The -f and -c arguments are deprecated, and not compatible "
                "with the -j argument, which should specify the same "
                "information. Either merge the frequencies and clusters data "
                "into the JSONL-formatted file (recommended), or use only the "
                "-f and -c files, without the other lexical attributes.",
            )
        jsonl_loc = ensure_path(jsonl_loc)
        lex_attrs = srsly.read_jsonl(jsonl_loc)
    else:
        clusters_loc = ensure_path(clusters_loc)
        freqs_loc = ensure_path(freqs_loc)
        if freqs_loc is not None and not freqs_loc.exists():
            msg.fail("Can't find words frequencies file", freqs_loc, exits=1)
        lex_attrs = read_attrs_from_deprecated(freqs_loc, clusters_loc)

    with msg.loading("Creating model..."):
        nlp = create_model(lang, lex_attrs, name=model_name, base_model=base_model)

    # Create empty extra lexeme tables so the data from spacy-lookups-data
    # isn't loaded if these features are accessed
    if omit_extra_lookups:
        nlp.vocab.lookups_extra = Lookups()
        nlp.vocab.lookups_extra.add_table("lexeme_cluster")
        nlp.vocab.lookups_extra.add_table("lexeme_prob")
        nlp.vocab.lookups_extra.add_table("lexeme_settings")

    msg.good("Successfully created model")
    if vectors_loc is not None:
        add_vectors(nlp, vectors_loc, truncate_vectors, prune_vectors, vectors_name)
    vec_added = len(nlp.vocab.vectors)
    lex_added = len(nlp.vocab)
    msg.good(
        "Sucessfully compiled vocab",
        "{} entries, {} vectors".format(lex_added, vec_added),
    )
    if not output_dir.exists():
        output_dir.mkdir()
    nlp.to_disk(output_dir)
    return nlp


def open_file(loc):
    """Handle .gz, .tar.gz or unzipped files"""
    loc = ensure_path(loc)
    if tarfile.is_tarfile(str(loc)):
        return tarfile.open(str(loc), "r:gz")
    elif loc.parts[-1].endswith("gz"):
        return (line.decode("utf8") for line in gzip.open(str(loc), "r"))
    elif loc.parts[-1].endswith("zip"):
        zip_file = zipfile.ZipFile(str(loc))
        names = zip_file.namelist()
        file_ = zip_file.open(names[0])
        return (line.decode("utf8") for line in file_)
    else:
        return loc.open("r", encoding="utf8")


def read_attrs_from_deprecated(freqs_loc, clusters_loc):
    if freqs_loc is not None:
        with msg.loading("Counting frequencies..."):
            probs, _ = read_freqs(freqs_loc)
        msg.good("Counted frequencies")
    else:
        probs, _ = ({}, DEFAULT_OOV_PROB)  # noqa: F841
    if clusters_loc:
        with msg.loading("Reading clusters..."):
            clusters = read_clusters(clusters_loc)
        msg.good("Read clusters")
    else:
        clusters = {}
    lex_attrs = []
    sorted_probs = sorted(probs.items(), key=lambda item: item[1], reverse=True)
    if len(sorted_probs):
        for i, (word, prob) in tqdm(enumerate(sorted_probs)):
            attrs = {"orth": word, "id": i, "prob": prob}
            # Decode as a little-endian string, so that we can do & 15 to get
            # the first 4 bits. See _parse_features.pyx
            if word in clusters:
                attrs["cluster"] = int(clusters[word][::-1], 2)
            else:
                attrs["cluster"] = 0
            lex_attrs.append(attrs)
    return lex_attrs


def create_model(lang, lex_attrs, name=None, base_model=None):
    if base_model:
        nlp = load_model(base_model)
        # keep the tokenizer but remove any existing pipeline components due to
        # potentially conflicting vectors
        for pipe in nlp.pipe_names:
            nlp.remove_pipe(pipe)
    else:
        lang_class = get_lang_class(lang)
        nlp = lang_class()
    for lexeme in nlp.vocab:
        lexeme.rank = OOV_RANK
    for attrs in lex_attrs:
        if "settings" in attrs:
            continue
        lexeme = nlp.vocab[attrs["orth"]]
        lexeme.set_attrs(**attrs)
    if len(nlp.vocab):
        oov_prob = min(lex.prob for lex in nlp.vocab) - 1
    else:
        oov_prob = DEFAULT_OOV_PROB
    nlp.vocab.cfg.update({"oov_prob": oov_prob})
    if name:
        nlp.meta["name"] = name
    return nlp


def add_vectors(nlp, vectors_loc, truncate_vectors, prune_vectors, name=None):
    vectors_loc = ensure_path(vectors_loc)
    if vectors_loc and vectors_loc.parts[-1].endswith(".npz"):
        nlp.vocab.vectors = Vectors(data=numpy.load(vectors_loc.open("rb")))
        for lex in nlp.vocab:
            if lex.rank and lex.rank != OOV_RANK:
                nlp.vocab.vectors.add(lex.orth, row=lex.rank)
    else:
        if vectors_loc:
            with msg.loading("Reading vectors from {}".format(vectors_loc)):
                vectors_data, vector_keys = read_vectors(vectors_loc, truncate_vectors)
            msg.good("Loaded vectors from {}".format(vectors_loc))
        else:
            vectors_data, vector_keys = (None, None)
        if vector_keys is not None:
            for word in vector_keys:
                if word not in nlp.vocab:
                    nlp.vocab[word]
        if vectors_data is not None:
            nlp.vocab.vectors = Vectors(data=vectors_data, keys=vector_keys)
    if name is None:
        nlp.vocab.vectors.name = "%s_model.vectors" % nlp.meta["lang"]
    else:
        nlp.vocab.vectors.name = name
    nlp.meta["vectors"]["name"] = nlp.vocab.vectors.name
    if prune_vectors >= 1:
        nlp.vocab.prune_vectors(prune_vectors)


def read_vectors(vectors_loc, truncate_vectors=0):
    f = open_file(vectors_loc)
    shape = tuple(int(size) for size in next(f).split())
    if truncate_vectors >= 1:
        shape = (truncate_vectors, shape[1])
    vectors_data = numpy.zeros(shape=shape, dtype="f")
    vectors_keys = []
    for i, line in enumerate(tqdm(f)):
        line = line.rstrip()
        pieces = line.rsplit(" ", vectors_data.shape[1])
        word = pieces.pop(0)
        if len(pieces) != vectors_data.shape[1]:
            msg.fail(Errors.E094.format(line_num=i, loc=vectors_loc), exits=1)
        vectors_data[i] = numpy.asarray(pieces, dtype="f")
        vectors_keys.append(word)
        if i == truncate_vectors - 1:
            break
    return vectors_data, vectors_keys


def read_freqs(freqs_loc, max_length=100, min_doc_freq=5, min_freq=50):
    counts = PreshCounter()
    total = 0
    with freqs_loc.open() as f:
        for i, line in enumerate(f):
            freq, doc_freq, key = line.rstrip().split("\t", 2)
            freq = int(freq)
            counts.inc(i + 1, freq)
            total += freq
    counts.smooth()
    log_total = math.log(total)
    probs = {}
    with freqs_loc.open() as f:
        for line in tqdm(f):
            freq, doc_freq, key = line.rstrip().split("\t", 2)
            doc_freq = int(doc_freq)
            freq = int(freq)
            if doc_freq >= min_doc_freq and freq >= min_freq and len(key) < max_length:
                try:
                    word = literal_eval(key)
                except SyntaxError:
                    # Take odd strings literally.
                    word = literal_eval("'%s'" % key)
                smooth_count = counts.smoother(int(freq))
                probs[word] = math.log(smooth_count) - log_total
    oov_prob = math.log(counts.smoother(0)) - log_total
    return probs, oov_prob


def read_clusters(clusters_loc):
    clusters = {}
    if ftfy is None:
        warnings.warn(Warnings.W004)
    with clusters_loc.open() as f:
        for line in tqdm(f):
            try:
                cluster, word, freq = line.split()
                if ftfy is not None:
                    word = ftfy.fix_text(word)
            except ValueError:
                continue
            # If the clusterer has only seen the word a few times, its
            # cluster is unreliable.
            if int(freq) >= 3:
                clusters[word] = cluster
            else:
                clusters[word] = "0"
    # Expand clusters with re-casing
    for word, cluster in list(clusters.items()):
        if word.lower() not in clusters:
            clusters[word.lower()] = cluster
        if word.title() not in clusters:
            clusters[word.title()] = cluster
        if word.upper() not in clusters:
            clusters[word.upper()] = cluster
    return clusters



# coding: utf8
from __future__ import unicode_literals

from ...lemmatizer import Lemmatizer
from ...symbols import POS, NOUN, VERB, ADJ, ADV, PRON, DET, AUX, PUNCT, ADP
from ...symbols import SCONJ, CCONJ
from ...symbols import VerbForm_inf, VerbForm_none, Number_sing, Degree_pos


class FrenchLemmatizer(Lemmatizer):
    """
    French language lemmatizer applies the default rule based lemmatization
    procedure with some modifications for better French language support.

    The parts of speech 'ADV', 'PRON', 'DET', 'ADP' and 'AUX' are added to use
    the rule-based lemmatization. As a last resort, the lemmatizer checks in
    the lookup table.
    """

    def __call__(self, string, univ_pos, morphology=None):
        lookup_table = self.lookups.get_table("lemma_lookup", {})
        if "lemma_rules" not in self.lookups:
            return [lookup_table.get(string, string)]
        if univ_pos in (NOUN, "NOUN", "noun"):
            univ_pos = "noun"
        elif univ_pos in (VERB, "VERB", "verb"):
            univ_pos = "verb"
        elif univ_pos in (ADJ, "ADJ", "adj"):
            univ_pos = "adj"
        elif univ_pos in (ADP, "ADP", "adp"):
            univ_pos = "adp"
        elif univ_pos in (ADV, "ADV", "adv"):
            univ_pos = "adv"
        elif univ_pos in (AUX, "AUX", "aux"):
            univ_pos = "aux"
        elif univ_pos in (CCONJ, "CCONJ", "cconj"):
            univ_pos = "cconj"
        elif univ_pos in (DET, "DET", "det"):
            univ_pos = "det"
        elif univ_pos in (PRON, "PRON", "pron"):
            univ_pos = "pron"
        elif univ_pos in (PUNCT, "PUNCT", "punct"):
            univ_pos = "punct"
        elif univ_pos in (SCONJ, "SCONJ", "sconj"):
            univ_pos = "sconj"
        else:
            return [self.lookup(string)]
        index_table = self.lookups.get_table("lemma_index", {})
        exc_table = self.lookups.get_table("lemma_exc", {})
        rules_table = self.lookups.get_table("lemma_rules", {})
        lemmas = self.lemmatize(
            string,
            index_table.get(univ_pos, {}),
            exc_table.get(univ_pos, {}),
            rules_table.get(univ_pos, []),
        )
        return lemmas

    def noun(self, string, morphology=None):
        return self(string, "noun", morphology)

    def verb(self, string, morphology=None):
        return self(string, "verb", morphology)

    def adj(self, string, morphology=None):
        return self(string, "adj", morphology)

    def punct(self, string, morphology=None):
        return self(string, "punct", morphology)

    def lookup(self, string, orth=None):
        lookup_table = self.lookups.get_table("lemma_lookup", {})
        if orth is not None and orth in lookup_table:
            return lookup_table[orth][0]
        return string

    def lemmatize(self, string, index, exceptions, rules):
        lookup_table = self.lookups.get_table("lemma_lookup", {})
        string = string.lower()
        forms = []
        if string in index:
            forms.append(string)
            return forms
        forms.extend(exceptions.get(string, []))
        oov_forms = []
        if not forms:
            for old, new in rules:
                if string.endswith(old):
                    form = string[: len(string) - len(old)] + new
                    if not form:
                        pass
                    elif form in index or not form.isalpha():
                        forms.append(form)
                    else:
                        oov_forms.append(form)
        if not forms:
            forms.extend(oov_forms)
        if not forms and string in lookup_table.keys():
            forms.append(lookup_table[string][0])
        if not forms:
            forms.append(string)
        return list(set(forms))



# coding: utf8
from __future__ import unicode_literals

from ..char_classes import LIST_ELLIPSES, LIST_ICONS, ALPHA, ALPHA_LOWER, ALPHA_UPPER

ELISION = " ' ’ ".strip().replace(" ", "")

abbrev = ("d", "D")

_infixes = (
    LIST_ELLIPSES
    + LIST_ICONS
    + [
        r"(?<=^[{ab}][{el}])(?=[{a}])".format(ab=abbrev, a=ALPHA, el=ELISION),
        r"(?<=[{al}])\.(?=[{au}])".format(al=ALPHA_LOWER, au=ALPHA_UPPER),
        r"(?<=[{a}])[,!?](?=[{a}])".format(a=ALPHA),
        r"(?<=[{a}])[:<>=](?=[{a}])".format(a=ALPHA),
        r"(?<=[{a}]),(?=[{a}])".format(a=ALPHA),
        r"(?<=[{a}])--(?=[{a}])".format(a=ALPHA),
        r"(?<=[0-9])-(?=[0-9])",
    ]
)

TOKENIZER_INFIXES = _infixes



# coding: utf8
from __future__ import unicode_literals

import pytest

import spacy


@pytest.fixture
def nlp():
    return spacy.blank("en")


def test_issue4924(nlp):
    docs_golds = [("", {})]
    nlp.evaluate(docs_golds)



# coding: utf8
from __future__ import unicode_literals

from ...attrs import LIKE_NUM


# Thirteen, fifteen etc. are written separate: on üç

_num_words = [
    "bir",
    "iki",
    "üç",
    "dört",
    "beş",
    "altı",
    "yedi",
    "sekiz",
    "dokuz",
    "on",
    "yirmi",
    "otuz",
    "kırk",
    "elli",
    "altmış",
    "yetmiş",
    "seksen",
    "doksan",
    "yüz",
    "bin",
    "milyon",
    "milyar",
    "trilyon",
    "katrilyon",
    "kentilyon",
]


_ordinal_words = [
    "birinci",
    "ikinci",
    "üçüncü",
    "dördüncü",
    "beşinci",
    "altıncı",
    "yedinci",
    "sekizinci",
    "dokuzuncu",
    "onuncu",
    "yirminci",
    "otuzuncu",
    "kırkıncı",
    "ellinci",
    "altmışıncı",
    "yetmişinci",
    "sekseninci",
    "doksanıncı",
    "yüzüncü",
    "bininci",
    "mliyonuncu",
    "milyarıncı",
    "trilyonuncu",
    "katrilyonuncu",
    "kentilyonuncu",
]

_ordinal_endings = ("inci", "ıncı", "nci", "ncı", "uncu", "üncü")

def like_num(text):
    if text.startswith(("+", "-", "±", "~")):
        text = text[1:]
    text = text.replace(",", "").replace(".", "")
    if text.isdigit():
        return True
    if text.count("/") == 1:
        num, denom = text.split("/")
        if num.isdigit() and denom.isdigit():
            return True

    text_lower = text.lower()

    #Check cardinal number
    if text_lower in _num_words:
        return True

    #Check ordinal number
    if text_lower in _ordinal_words:
        return True
    if text_lower.endswith(_ordinal_endings):
        if text_lower[:-3].isdigit() or text_lower[:-4].isdigit():
            return True

    return False


LEX_ATTRS = {LIKE_NUM: like_num}



# coding: utf8
from __future__ import unicode_literals

import warnings

from collections import OrderedDict
from wasabi import Printer

from .tokens import Doc, Token, Span
from .errors import Errors, Warnings


def analyze_pipes(pipeline, name, pipe, index, warn=True):
    """Analyze a pipeline component with respect to its position in the current
    pipeline and the other components. Will check whether requirements are
    fulfilled (e.g. if previous components assign the attributes).

    pipeline (list): A list of (name, pipe) tuples e.g. nlp.pipeline.
    name (unicode): The name of the pipeline component to analyze.
    pipe (callable): The pipeline component function to analyze.
    index (int): The index of the component in the pipeline.
    warn (bool): Show user warning if problem is found.
    RETURNS (list): The problems found for the given pipeline component.
    """
    assert pipeline[index][0] == name
    prev_pipes = pipeline[:index]
    pipe_requires = getattr(pipe, "requires", [])
    requires = OrderedDict([(annot, False) for annot in pipe_requires])
    if requires:
        for prev_name, prev_pipe in prev_pipes:
            prev_assigns = getattr(prev_pipe, "assigns", [])
            for annot in prev_assigns:
                requires[annot] = True
    problems = []
    for annot, fulfilled in requires.items():
        if not fulfilled:
            problems.append(annot)
            if warn:
                warnings.warn(Warnings.W025.format(name=name, attr=annot))
    return problems


def analyze_all_pipes(pipeline, warn=True):
    """Analyze all pipes in the pipeline in order.

    pipeline (list): A list of (name, pipe) tuples e.g. nlp.pipeline.
    warn (bool): Show user warning if problem is found.
    RETURNS (dict): The problems found, keyed by component name.
    """
    problems = {}
    for i, (name, pipe) in enumerate(pipeline):
        problems[name] = analyze_pipes(pipeline, name, pipe, i, warn=warn)
    return problems


def dot_to_dict(values):
    """Convert dot notation to a dict. For example: ["token.pos", "token._.xyz"]
    become {"token": {"pos": True, "_": {"xyz": True }}}.

    values (iterable): The values to convert.
    RETURNS (dict): The converted values.
    """
    result = {}
    for value in values:
        path = result
        parts = value.lower().split(".")
        for i, item in enumerate(parts):
            is_last = i == len(parts) - 1
            path = path.setdefault(item, True if is_last else {})
    return result


def validate_attrs(values):
    """Validate component attributes provided to "assigns", "requires" etc.
    Raises error for invalid attributes and formatting. Doesn't check if
    custom extension attributes are registered, since this is something the
    user might want to do themselves later in the component.

    values (iterable): The string attributes to check, e.g. `["token.pos"]`.
    RETURNS (iterable): The checked attributes.
    """
    data = dot_to_dict(values)
    objs = {"doc": Doc, "token": Token, "span": Span}
    for obj_key, attrs in data.items():
        if obj_key == "span":
            # Support Span only for custom extension attributes
            span_attrs = [attr for attr in values if attr.startswith("span.")]
            span_attrs = [attr for attr in span_attrs if not attr.startswith("span._.")]
            if span_attrs:
                raise ValueError(Errors.E180.format(attrs=", ".join(span_attrs)))
        if obj_key not in objs:  # first element is not doc/token/span
            invalid_attrs = ", ".join(a for a in values if a.startswith(obj_key))
            raise ValueError(Errors.E181.format(obj=obj_key, attrs=invalid_attrs))
        if not isinstance(attrs, dict):  # attr is something like "doc"
            raise ValueError(Errors.E182.format(attr=obj_key))
        for attr, value in attrs.items():
            if attr == "_":
                if value is True:  # attr is something like "doc._"
                    raise ValueError(Errors.E182.format(attr="{}._".format(obj_key)))
                for ext_attr, ext_value in value.items():
                    # We don't check whether the attribute actually exists
                    if ext_value is not True:  # attr is something like doc._.x.y
                        good = "{}._.{}".format(obj_key, ext_attr)
                        bad = "{}.{}".format(good, ".".join(ext_value))
                        raise ValueError(Errors.E183.format(attr=bad, solution=good))
                continue  # we can't validate those further
            if attr.endswith("_"):  # attr is something like "token.pos_"
                raise ValueError(Errors.E184.format(attr=attr, solution=attr[:-1]))
            if value is not True:  # attr is something like doc.x.y
                good = "{}.{}".format(obj_key, attr)
                bad = "{}.{}".format(good, ".".join(value))
                raise ValueError(Errors.E183.format(attr=bad, solution=good))
            obj = objs[obj_key]
            if not hasattr(obj, attr):
                raise ValueError(Errors.E185.format(obj=obj_key, attr=attr))
    return values


def _get_feature_for_attr(pipeline, attr, feature):
    assert feature in ["assigns", "requires"]
    result = []
    for pipe_name, pipe in pipeline:
        pipe_assigns = getattr(pipe, feature, [])
        if attr in pipe_assigns:
            result.append((pipe_name, pipe))
    return result


def get_assigns_for_attr(pipeline, attr):
    """Get all pipeline components that assign an attr, e.g. "doc.tensor".

    pipeline (list): A list of (name, pipe) tuples e.g. nlp.pipeline.
    attr (unicode): The attribute to check.
    RETURNS (list): (name, pipeline) tuples of components that assign the attr.
    """
    return _get_feature_for_attr(pipeline, attr, "assigns")


def get_requires_for_attr(pipeline, attr):
    """Get all pipeline components that require an attr, e.g. "doc.tensor".

    pipeline (list): A list of (name, pipe) tuples e.g. nlp.pipeline.
    attr (unicode): The attribute to check.
    RETURNS (list): (name, pipeline) tuples of components that require the attr.
    """
    return _get_feature_for_attr(pipeline, attr, "requires")


def print_summary(nlp, pretty=True, no_print=False):
    """Print a formatted summary for the current nlp object's pipeline. Shows
    a table with the pipeline components and why they assign and require, as
    well as any problems if available.

    nlp (Language): The nlp object.
    pretty (bool): Pretty-print the results (color etc).
    no_print (bool): Don't print anything, just return the data.
    RETURNS (dict): A dict with "overview" and "problems".
    """
    msg = Printer(pretty=pretty, no_print=no_print)
    overview = []
    problems = {}
    for i, (name, pipe) in enumerate(nlp.pipeline):
        requires = getattr(pipe, "requires", [])
        assigns = getattr(pipe, "assigns", [])
        retok = getattr(pipe, "retokenizes", False)
        overview.append((i, name, requires, assigns, retok))
        problems[name] = analyze_pipes(nlp.pipeline, name, pipe, i, warn=False)
    msg.divider("Pipeline Overview")
    header = ("#", "Component", "Requires", "Assigns", "Retokenizes")
    msg.table(overview, header=header, divider=True, multiline=True)
    n_problems = sum(len(p) for p in problems.values())
    if any(p for p in problems.values()):
        msg.divider("Problems ({})".format(n_problems))
        for name, problem in problems.items():
            if problem:
                problem = ", ".join(problem)
                msg.warn("'{}' requirements not met: {}".format(name, problem))
    else:
        msg.good("No problems found.")
    if no_print:
        return {"overview": overview, "problems": problems}



# coding: utf8
from __future__ import unicode_literals

from ...symbols import NOUN, PROPN, PRON, VERB, AUX
from ...errors import Errors


def noun_chunks(doclike):
    doc = doclike.doc

    if not doc.is_parsed:
        raise ValueError(Errors.E029)

    if not len(doc):
        return
    np_label = doc.vocab.strings.add("NP")
    left_labels = ["det", "fixed", "neg"]  # ['nunmod', 'det', 'appos', 'fixed']
    right_labels = ["flat", "fixed", "compound", "neg"]
    stop_labels = ["punct"]
    np_left_deps = [doc.vocab.strings.add(label) for label in left_labels]
    np_right_deps = [doc.vocab.strings.add(label) for label in right_labels]
    stop_deps = [doc.vocab.strings.add(label) for label in stop_labels]
    for token in doclike:
        if token.pos in [PROPN, NOUN, PRON]:
            left, right = noun_bounds(
                doc, token, np_left_deps, np_right_deps, stop_deps
            )
            yield left.i, right.i + 1, np_label
            token = right
        token = next_token(token)


def is_verb_token(token):
    return token.pos in [VERB, AUX]


def next_token(token):
    try:
        return token.nbor()
    except IndexError:
        return None


def noun_bounds(doc, root, np_left_deps, np_right_deps, stop_deps):
    left_bound = root
    for token in reversed(list(root.lefts)):
        if token.dep in np_left_deps:
            left_bound = token
    right_bound = root
    for token in root.rights:
        if token.dep in np_right_deps:
            left, right = noun_bounds(
                doc, token, np_left_deps, np_right_deps, stop_deps
            )
            if list(
                filter(
                    lambda t: is_verb_token(t) or t.dep in stop_deps,
                    doc[left_bound.i : right.i],
                )
            ):
                break
            else:
                right_bound = right
    return left_bound, right_bound


SYNTAX_ITERATORS = {"noun_chunks": noun_chunks}



# coding: utf8
from __future__ import unicode_literals

from ...language import Language
from ...attrs import LANG, NORM
from ...util import update_exc, add_lookups
from ..norm_exceptions import BASE_NORMS
from .stop_words import STOP_WORDS
from .lex_attrs import LEX_ATTRS
from .tokenizer_exceptions import TOKENIZER_EXCEPTIONS
from .tag_map import TAG_MAP
from .punctuation import TOKENIZER_SUFFIXES
from .syntax_iterators import SYNTAX_ITERATORS


class PersianDefaults(Language.Defaults):
    lex_attr_getters = dict(Language.Defaults.lex_attr_getters)
    lex_attr_getters.update(LEX_ATTRS)
    lex_attr_getters[NORM] = add_lookups(
        Language.Defaults.lex_attr_getters[NORM], BASE_NORMS
    )
    lex_attr_getters[LANG] = lambda text: "fa"
    tokenizer_exceptions = update_exc(TOKENIZER_EXCEPTIONS)
    stop_words = STOP_WORDS
    tag_map = TAG_MAP
    suffixes = TOKENIZER_SUFFIXES
    writing_system = {"direction": "rtl", "has_case": False, "has_letters": True}
    syntax_iterators = SYNTAX_ITERATORS


class Persian(Language):
    lang = "fa"
    Defaults = PersianDefaults


__all__ = ["Persian"]



# coding: utf8
from __future__ import unicode_literals

"""
Example sentences to test spaCy and its language models.
>>> from spacy.lang.hy.examples import sentences
>>> docs = nlp.pipe(sentences)
"""


sentences = [
    "Լոնդոնը Միացյալ Թագավորության մեծ քաղաք է։",
    "Ո՞վ է Ֆրանսիայի նախագահը։",
    "Ո՞րն է Միացյալ Նահանգների մայրաքաղաքը։",
    "Ե՞րբ է ծնվել Բարաք Օբաման։",
]



# coding: utf8
from __future__ import unicode_literals


def add_codes(err_cls):
    """Add error codes to string messages via class attribute names."""

    class ErrorsWithCodes(err_cls):
        def __getattribute__(self, code):
            msg = super(ErrorsWithCodes, self).__getattribute__(code)
            if code.startswith("__"):  # python system attributes like __class__
                return msg
            else:
                return "[{code}] {msg}".format(code=code, msg=msg)

    return ErrorsWithCodes()


# fmt: off

@add_codes
class Warnings(object):
    W001 = ("As of spaCy v2.0, the keyword argument `path=` is deprecated. "
            "You can now call spacy.load with the path as its first argument, "
            "and the model's meta.json will be used to determine the language "
            "to load. For example:\nnlp = spacy.load('{path}')")
    W002 = ("Tokenizer.from_list is now deprecated. Create a new Doc object "
            "instead and pass in the strings as the `words` keyword argument, "
            "for example:\nfrom spacy.tokens import Doc\n"
            "doc = Doc(nlp.vocab, words=[...])")
    W003 = ("Positional arguments to Doc.merge are deprecated. Instead, use "
            "the keyword arguments, for example tag=, lemma= or ent_type=.")
    W004 = ("No text fixing enabled. Run `pip install ftfy` to enable fixing "
            "using ftfy.fix_text if necessary.")
    W005 = ("Doc object not parsed. This means displaCy won't be able to "
            "generate a dependency visualization for it. Make sure the Doc "
            "was processed with a model that supports dependency parsing, and "
            "not just a language class like `English()`. For more info, see "
            "the docs:\nhttps://spacy.io/usage/models")
    W006 = ("No entities to visualize found in Doc object. If this is "
            "surprising to you, make sure the Doc was processed using a model "
            "that supports named entity recognition, and check the `doc.ents` "
            "property manually if necessary.")
    W007 = ("The model you're using has no word vectors loaded, so the result "
            "of the {obj}.similarity method will be based on the tagger, "
            "parser and NER, which may not give useful similarity judgements. "
            "This may happen if you're using one of the small models, e.g. "
            "`en_core_web_sm`, which don't ship with word vectors and only "
            "use context-sensitive tensors. You can always add your own word "
            "vectors, or use one of the larger models instead if available.")
    W008 = ("Evaluating {obj}.similarity based on empty vectors.")
    W009 = ("Custom factory '{name}' provided by entry points of another "
            "package overwrites built-in factory.")
    W010 = ("As of v2.1.0, the PhraseMatcher doesn't have a phrase length "
            "limit anymore, so the max_length argument is now deprecated. "
            "If you did not specify this parameter, make sure you call the "
            "constructor with named arguments instead of positional ones.")
    W011 = ("It looks like you're calling displacy.serve from within a "
            "Jupyter notebook or a similar environment. This likely means "
            "you're already running a local web server, so there's no need to "
            "make displaCy start another one. Instead, you should be able to "
            "replace displacy.serve with displacy.render to show the "
            "visualization.")
    W012 = ("A Doc object you're adding to the PhraseMatcher for pattern "
            "'{key}' is parsed and/or tagged, but to match on '{attr}', you "
            "don't actually need this information. This means that creating "
            "the patterns is potentially much slower, because all pipeline "
            "components are applied. To only create tokenized Doc objects, "
            "try using `nlp.make_doc(text)` or process all texts as a stream "
            "using `list(nlp.tokenizer.pipe(all_texts))`.")
    W013 = ("As of v2.1.0, {obj}.merge is deprecated. Please use the more "
            "efficient and less error-prone Doc.retokenize context manager "
            "instead.")
    W014 = ("As of v2.1.0, the `disable` keyword argument on the serialization "
            "methods is and should be replaced with `exclude`. This makes it "
            "consistent with the other serializable objects.")
    W015 = ("As of v2.1.0, the use of keyword arguments to exclude fields from "
            "being serialized or deserialized is deprecated. Please use the "
            "`exclude` argument instead. For example: exclude=['{arg}'].")
    W016 = ("The keyword argument `n_threads` is now deprecated. As of v2.2.2, "
            "the argument `n_process` controls parallel inference via "
            "multiprocessing.")
    W017 = ("Alias '{alias}' already exists in the Knowledge Base.")
    W018 = ("Entity '{entity}' already exists in the Knowledge Base - "
            "ignoring the duplicate entry.")
    W019 = ("Changing vectors name from {old} to {new}, to avoid clash with "
            "previously loaded vectors. See Issue #3853.")
    W020 = ("Unnamed vectors. This won't allow multiple vectors models to be "
            "loaded. (Shape: {shape})")
    W021 = ("Unexpected hash collision in PhraseMatcher. Matches may be "
            "incorrect. Modify PhraseMatcher._terminal_hash to fix.")
    W022 = ("Training a new part-of-speech tagger using a model with no "
            "lemmatization rules or data. This means that the trained model "
            "may not be able to lemmatize correctly. If this is intentional "
            "or the language you're using doesn't have lemmatization data, "
            "please ignore this warning. If this is surprising, make sure you "
            "have the spacy-lookups-data package installed.")
    W023 = ("Multiprocessing of Language.pipe is not supported in Python 2. "
            "'n_process' will be set to 1.")
    W024 = ("Entity '{entity}' - Alias '{alias}' combination already exists in "
            "the Knowledge Base.")
    W025 = ("'{name}' requires '{attr}' to be assigned, but none of the "
            "previous components in the pipeline declare that they assign it.")
    W026 = ("Unable to set all sentence boundaries from dependency parses.")
    W027 = ("Found a large training file of {size} bytes. Note that it may "
            "be more efficient to split your training data into multiple "
            "smaller JSON files instead.")
    W028 = ("Doc.from_array was called with a vector of type '{type}', "
            "but is expecting one of type 'uint64' instead. This may result "
            "in problems with the vocab further on in the pipeline.")
    W029 = ("Unable to align tokens with entities from character offsets. "
            "Discarding entity annotation for the text: {text}.")
    W030 = ("Some entities could not be aligned in the text \"{text}\" with "
            "entities \"{entities}\". Use "
            "`spacy.gold.biluo_tags_from_offsets(nlp.make_doc(text), entities)`"
            " to check the alignment. Misaligned entities (with BILUO tag '-') "
            "will be ignored during training.")
    W031 = ("Model '{model}' ({model_version}) requires spaCy {version} and "
            "is incompatible with the current spaCy version ({current}). This "
            "may lead to unexpected results or runtime errors. To resolve "
            "this, download a newer compatible model or retrain your custom "
            "model with the current spaCy version. For more details and "
            "available updates, run: python -m spacy validate")
    W032 = ("Unable to determine model compatibility for model '{model}' "
            "({model_version}) with the current spaCy version ({current}). "
            "This may lead to unexpected results or runtime errors. To resolve "
            "this, download a newer compatible model or retrain your custom "
            "model with the current spaCy version. For more details and "
            "available updates, run: python -m spacy validate")
    W033 = ("Training a new {model} using a model with an empty lexeme "
            "normalization table. This may degrade the performance to some "
            "degree. If this is intentional or this language doesn't have a "
            "normalization table, please ignore this warning.")
    W034 = ("Please install the package spacy-lookups-data in order to include "
            "the default lexeme normalization table for the language '{lang}'.")
    W035 = ('Discarding subpattern "{pattern}" due to an unrecognized '
            "attribute or operator.")


@add_codes
class Errors(object):
    E001 = ("No component '{name}' found in pipeline. Available names: {opts}")
    E002 = ("Can't find factory for '{name}'. This usually happens when spaCy "
            "calls `nlp.create_pipe` with a component name that's not built "
            "in - for example, when constructing the pipeline from a model's "
            "meta.json. If you're using a custom component, you can write to "
            "`Language.factories['{name}']` or remove it from the model meta "
            "and add it via `nlp.add_pipe` instead.")
    E003 = ("Not a valid pipeline component. Expected callable, but "
            "got {component} (name: '{name}').")
    E004 = ("If you meant to add a built-in component, use `create_pipe`: "
            "`nlp.add_pipe(nlp.create_pipe('{component}'))`")
    E005 = ("Pipeline component '{name}' returned None. If you're using a "
            "custom component, maybe you forgot to return the processed Doc?")
    E006 = ("Invalid constraints. You can only set one of the following: "
            "before, after, first, last.")
    E007 = ("'{name}' already exists in pipeline. Existing names: {opts}")
    E008 = ("Some current components would be lost when restoring previous "
            "pipeline state. If you added components after calling "
            "`nlp.disable_pipes()`, you should remove them explicitly with "
            "`nlp.remove_pipe()` before the pipeline is restored. Names of "
            "the new components: {names}")
    E009 = ("The `update` method expects same number of docs and golds, but "
            "got: {n_docs} docs, {n_golds} golds.")
    E010 = ("Word vectors set to length 0. This may be because you don't have "
            "a model installed or loaded, or because your model doesn't "
            "include word vectors. For more info, see the docs:\n"
            "https://spacy.io/usage/models")
    E011 = ("Unknown operator: '{op}'. Options: {opts}")
    E012 = ("Cannot add pattern for zero tokens to matcher.\nKey: {key}")
    E013 = ("Error selecting action in matcher")
    E014 = ("Unknown tag ID: {tag}")
    E015 = ("Conflicting morphology exception for ({tag}, {orth}). Use "
            "`force=True` to overwrite.")
    E016 = ("MultitaskObjective target should be function or one of: dep, "
            "tag, ent, dep_tag_offset, ent_tag.")
    E017 = ("Can only add unicode or bytes. Got type: {value_type}")
    E018 = ("Can't retrieve string for hash '{hash_value}'. This usually "
            "refers to an issue with the `Vocab` or `StringStore`.")
    E019 = ("Can't create transition with unknown action ID: {action}. Action "
            "IDs are enumerated in spacy/syntax/{src}.pyx.")
    E020 = ("Could not find a gold-standard action to supervise the "
            "dependency parser. The tree is non-projective (i.e. it has "
            "crossing arcs - see spacy/syntax/nonproj.pyx for definitions). "
            "The ArcEager transition system only supports projective trees. "
            "To learn non-projective representations, transform the data "
            "before training and after parsing. Either pass "
            "`make_projective=True` to the GoldParse class, or use "
            "spacy.syntax.nonproj.preprocess_training_data.")
    E021 = ("Could not find a gold-standard action to supervise the "
            "dependency parser. The GoldParse was projective. The transition "
            "system has {n_actions} actions. State at failure: {state}")
    E022 = ("Could not find a transition with the name '{name}' in the NER "
            "model.")
    E023 = ("Error cleaning up beam: The same state occurred twice at "
            "memory address {addr} and position {i}.")
    E024 = ("Could not find an optimal move to supervise the parser. Usually, "
            "this means that the model can't be updated in a way that's valid "
            "and satisfies the correct annotations specified in the GoldParse. "
            "For example, are all labels added to the model? If you're "
            "training a named entity recognizer, also make sure that none of "
            "your annotated entity spans have leading or trailing whitespace "
            "or punctuation. "
            "You can also use the experimental `debug-data` command to "
            "validate your JSON-formatted training data. For details, run:\n"
            "python -m spacy debug-data --help")
    E025 = ("String is too long: {length} characters. Max is 2**30.")
    E026 = ("Error accessing token at position {i}: out of bounds in Doc of "
            "length {length}.")
    E027 = ("Arguments 'words' and 'spaces' should be sequences of the same "
            "length, or 'spaces' should be left default at None. spaces "
            "should be a sequence of booleans, with True meaning that the "
            "word owns a ' ' character following it.")
    E028 = ("orths_and_spaces expects either a list of unicode string or a "
            "list of (unicode, bool) tuples. Got bytes instance: {value}")
    E029 = ("noun_chunks requires the dependency parse, which requires a "
            "statistical model to be installed and loaded. For more info, see "
            "the documentation:\nhttps://spacy.io/usage/models")
    E030 = ("Sentence boundaries unset. You can add the 'sentencizer' "
            "component to the pipeline with: "
            "nlp.add_pipe(nlp.create_pipe('sentencizer')) "
            "Alternatively, add the dependency parser, or set sentence "
            "boundaries by setting doc[i].is_sent_start.")
    E031 = ("Invalid token: empty string ('') at position {i}.")
    E032 = ("Conflicting attributes specified in doc.from_array(): "
            "(HEAD, SENT_START). The HEAD attribute currently sets sentence "
            "boundaries implicitly, based on the tree structure. This means "
            "the HEAD attribute would potentially override the sentence "
            "boundaries set by SENT_START.")
    E033 = ("Cannot load into non-empty Doc of length {length}.")
    E034 = ("Doc.merge received {n_args} non-keyword arguments. Expected "
            "either 3 arguments (deprecated), or 0 (use keyword arguments).\n"
            "Arguments supplied:\n{args}\nKeyword arguments:{kwargs}")
    E035 = ("Error creating span with start {start} and end {end} for Doc of "
            "length {length}.")
    E036 = ("Error calculating span: Can't find a token starting at character "
            "offset {start}.")
    E037 = ("Error calculating span: Can't find a token ending at character "
            "offset {end}.")
    E038 = ("Error finding sentence for span. Infinite loop detected.")
    E039 = ("Array bounds exceeded while searching for root word. This likely "
            "means the parse tree is in an invalid state. Please report this "
            "issue here: http://github.com/explosion/spaCy/issues")
    E040 = ("Attempt to access token at {i}, max length {max_length}.")
    E041 = ("Invalid comparison operator: {op}. Likely a Cython bug?")
    E042 = ("Error accessing doc[{i}].nbor({j}), for doc of length {length}.")
    E043 = ("Refusing to write to token.sent_start if its document is parsed, "
            "because this may cause inconsistent state.")
    E044 = ("Invalid value for token.sent_start: {value}. Must be one of: "
            "None, True, False")
    E045 = ("Possibly infinite loop encountered while looking for {attr}.")
    E046 = ("Can't retrieve unregistered extension attribute '{name}'. Did "
            "you forget to call the `set_extension` method?")
    E047 = ("Can't assign a value to unregistered extension attribute "
            "'{name}'. Did you forget to call the `set_extension` method?")
    E048 = ("Can't import language {lang} from spacy.lang: {err}")
    E049 = ("Can't find spaCy data directory: '{path}'. Check your "
            "installation and permissions, or use spacy.util.set_data_path "
            "to customise the location if necessary.")
    E050 = ("Can't find model '{name}'. It doesn't seem to be a shortcut "
            "link, a Python package or a valid path to a data directory.")
    E051 = ("Cant' load '{name}'. If you're using a shortcut link, make sure "
            "it points to a valid package (not just a data directory).")
    E052 = ("Can't find model directory: {path}")
    E053 = ("Could not read meta.json from {path}")
    E054 = ("No valid '{setting}' setting found in model meta.json.")
    E055 = ("Invalid ORTH value in exception:\nKey: {key}\nOrths: {orths}")
    E056 = ("Invalid tokenizer exception: ORTH values combined don't match "
            "original string.\nKey: {key}\nOrths: {orths}")
    E057 = ("Stepped slices not supported in Span objects. Try: "
            "list(tokens)[start:stop:step] instead.")
    E058 = ("Could not retrieve vector for key {key}.")
    E059 = ("One (and only one) keyword arg must be set. Got: {kwargs}")
    E060 = ("Cannot add new key to vectors: the table is full. Current shape: "
            "({rows}, {cols}).")
    E061 = ("Bad file name: {filename}. Example of a valid file name: "
            "'vectors.128.f.bin'")
    E062 = ("Cannot find empty bit for new lexical flag. All bits between 0 "
            "and 63 are occupied. You can replace one by specifying the "
            "`flag_id` explicitly, e.g. "
            "`nlp.vocab.add_flag(your_func, flag_id=IS_ALPHA`.")
    E063 = ("Invalid value for flag_id: {value}. Flag IDs must be between 1 "
            "and 63 (inclusive).")
    E064 = ("Error fetching a Lexeme from the Vocab. When looking up a "
            "string, the lexeme returned had an orth ID that did not match "
            "the query string. This means that the cached lexeme structs are "
            "mismatched to the string encoding table. The mismatched:\n"
            "Query string: {string}\nOrth cached: {orth}\nOrth ID: {orth_id}")
    E065 = ("Only one of the vector table's width and shape can be specified. "
            "Got width {width} and shape {shape}.")
    E066 = ("Error creating model helper for extracting columns. Can only "
            "extract columns by positive integer. Got: {value}.")
    E067 = ("Invalid BILUO tag sequence: Got a tag starting with 'I' (inside "
            "an entity) without a preceding 'B' (beginning of an entity). "
            "Tag sequence:\n{tags}")
    E068 = ("Invalid BILUO tag: '{tag}'.")
    E069 = ("Invalid gold-standard parse tree. Found cycle between word "
            "IDs: {cycle} (tokens: {cycle_tokens}) in the document starting "
            "with tokens: {doc_tokens}.")
    E070 = ("Invalid gold-standard data. Number of documents ({n_docs}) "
            "does not align with number of annotations ({n_annots}).")
    E071 = ("Error creating lexeme: specified orth ID ({orth}) does not "
            "match the one in the vocab ({vocab_orth}).")
    E072 = ("Error serializing lexeme: expected data length {length}, "
            "got {bad_length}.")
    E073 = ("Cannot assign vector of length {new_length}. Existing vectors "
            "are of length {length}. You can use `vocab.reset_vectors` to "
            "clear the existing vectors and resize the table.")
    E074 = ("Error interpreting compiled match pattern: patterns are expected "
            "to end with the attribute {attr}. Got: {bad_attr}.")
    E075 = ("Error accepting match: length ({length}) > maximum length "
            "({max_len}).")
    E076 = ("Error setting tensor on Doc: tensor has {rows} rows, while Doc "
            "has {words} words.")
    E077 = ("Error computing {value}: number of Docs ({n_docs}) does not "
            "equal number of GoldParse objects ({n_golds}) in batch.")
    E078 = ("Error computing score: number of words in Doc ({words_doc}) does "
            "not equal number of words in GoldParse ({words_gold}).")
    E079 = ("Error computing states in beam: number of predicted beams "
            "({pbeams}) does not equal number of gold beams ({gbeams}).")
    E080 = ("Duplicate state found in beam: {key}.")
    E081 = ("Error getting gradient in beam: number of histories ({n_hist}) "
            "does not equal number of losses ({losses}).")
    E082 = ("Error deprojectivizing parse: number of heads ({n_heads}), "
            "projective heads ({n_proj_heads}) and labels ({n_labels}) do not "
            "match.")
    E083 = ("Error setting extension: only one of `default`, `method`, or "
            "`getter` (plus optional `setter`) is allowed. Got: {nr_defined}")
    E084 = ("Error assigning label ID {label} to span: not in StringStore.")
    E085 = ("Can't create lexeme for string '{string}'.")
    E086 = ("Error deserializing lexeme '{string}': orth ID {orth_id} does "
            "not match hash {hash_id} in StringStore.")
    E087 = ("Unknown displaCy style: {style}.")
    E088 = ("Text of length {length} exceeds maximum of {max_length}. The "
            "v2.x parser and NER models require roughly 1GB of temporary "
            "memory per 100,000 characters in the input. This means long "
            "texts may cause memory allocation errors. If you're not using "
            "the parser or NER, it's probably safe to increase the "
            "`nlp.max_length` limit. The limit is in number of characters, so "
            "you can check whether your inputs are too long by checking "
            "`len(text)`.")
    E089 = ("Extensions can't have a setter argument without a getter "
            "argument. Check the keyword arguments on `set_extension`.")
    E090 = ("Extension '{name}' already exists on {obj}. To overwrite the "
            "existing extension, set `force=True` on `{obj}.set_extension`.")
    E091 = ("Invalid extension attribute {name}: expected callable or None, "
            "but got: {value}")
    E092 = ("Could not find or assign name for word vectors. Ususally, the "
            "name is read from the model's meta.json in vector.name. "
            "Alternatively, it is built from the 'lang' and 'name' keys in "
            "the meta.json. Vector names are required to avoid issue #1660.")
    E093 = ("token.ent_iob values make invalid sequence: I without B\n{seq}")
    E094 = ("Error reading line {line_num} in vectors file {loc}.")
    E095 = ("Can't write to frozen dictionary. This is likely an internal "
            "error. Are you writing to a default function argument?")
    E096 = ("Invalid object passed to displaCy: Can only visualize Doc or "
            "Span objects, or dicts if set to manual=True.")
    E097 = ("Invalid pattern: expected token pattern (list of dicts) or "
            "phrase pattern (string) but got:\n{pattern}")
    E098 = ("Invalid pattern specified: expected both SPEC and PATTERN.")
    E099 = ("First node of pattern should be a root node. The root should "
            "only contain NODE_NAME.")
    E100 = ("Nodes apart from the root should contain NODE_NAME, NBOR_NAME and "
            "NBOR_RELOP.")
    E101 = ("NODE_NAME should be a new node and NBOR_NAME should already have "
            "have been declared in previous edges.")
    E102 = ("Can't merge non-disjoint spans. '{token}' is already part of "
            "tokens to merge. If you want to find the longest non-overlapping "
            "spans, you can use the util.filter_spans helper:\n"
            "https://spacy.io/api/top-level#util.filter_spans")
    E103 = ("Trying to set conflicting doc.ents: '{span1}' and '{span2}'. A "
            "token can only be part of one entity, so make sure the entities "
            "you're setting don't overlap.")
    E104 = ("Can't find JSON schema for '{name}'.")
    E105 = ("The Doc.print_tree() method is now deprecated. Please use "
            "Doc.to_json() instead or write your own function.")
    E106 = ("Can't find doc._.{attr} attribute specified in the underscore "
            "settings: {opts}")
    E107 = ("Value of doc._.{attr} is not JSON-serializable: {value}")
    E108 = ("As of spaCy v2.1, the pipe name `sbd` has been deprecated "
            "in favor of the pipe name `sentencizer`, which does the same "
            "thing. For example, use `nlp.create_pipeline('sentencizer')`")
    E109 = ("Model for component '{name}' not initialized. Did you forget to "
            "load a model, or forget to call begin_training()?")
    E110 = ("Invalid displaCy render wrapper. Expected callable, got: {obj}")
    E111 = ("Pickling a token is not supported, because tokens are only views "
            "of the parent Doc and can't exist on their own. A pickled token "
            "would always have to include its Doc and Vocab, which has "
            "practically no advantage over pickling the parent Doc directly. "
            "So instead of pickling the token, pickle the Doc it belongs to.")
    E112 = ("Pickling a span is not supported, because spans are only views "
            "of the parent Doc and can't exist on their own. A pickled span "
            "would always have to include its Doc and Vocab, which has "
            "practically no advantage over pickling the parent Doc directly. "
            "So instead of pickling the span, pickle the Doc it belongs to or "
            "use Span.as_doc to convert the span to a standalone Doc object.")
    E113 = ("The newly split token can only have one root (head = 0).")
    E114 = ("The newly split token needs to have a root (head = 0).")
    E115 = ("All subtokens must have associated heads.")
    E116 = ("Cannot currently add labels to pretrained text classifier. Add "
            "labels before training begins. This functionality was available "
            "in previous versions, but had significant bugs that led to poor "
            "performance.")
    E117 = ("The newly split tokens must match the text of the original token. "
            "New orths: {new}. Old text: {old}.")
    E118 = ("The custom extension attribute '{attr}' is not registered on the "
            "Token object so it can't be set during retokenization. To "
            "register an attribute, use the Token.set_extension classmethod.")
    E119 = ("Can't set custom extension attribute '{attr}' during "
            "retokenization because it's not writable. This usually means it "
            "was registered with a getter function (and no setter) or as a "
            "method extension, so the value is computed dynamically. To "
            "overwrite a custom attribute manually, it should be registered "
            "with a default value or with a getter AND setter.")
    E120 = ("Can't set custom extension attributes during retokenization. "
            "Expected dict mapping attribute names to values, but got: {value}")
    E121 = ("Can't bulk merge spans. Attribute length {attr_len} should be "
            "equal to span length ({span_len}).")
    E122 = ("Cannot find token to be split. Did it get merged?")
    E123 = ("Cannot find head of token to be split. Did it get merged?")
    E124 = ("Cannot read from file: {path}. Supported formats: {formats}")
    E125 = ("Unexpected value: {value}")
    E126 = ("Unexpected matcher predicate: '{bad}'. Expected one of: {good}. "
            "This is likely a bug in spaCy, so feel free to open an issue.")
    E127 = ("Cannot create phrase pattern representation for length 0. This "
            "is likely a bug in spaCy.")
    E128 = ("Unsupported serialization argument: '{arg}'. The use of keyword "
            "arguments to exclude fields from being serialized or deserialized "
            "is now deprecated. Please use the `exclude` argument instead. "
            "For example: exclude=['{arg}'].")
    E129 = ("Cannot write the label of an existing Span object because a Span "
            "is a read-only view of the underlying Token objects stored in the "
            "Doc. Instead, create a new Span object and specify the `label` "
            "keyword argument, for example:\nfrom spacy.tokens import Span\n"
            "span = Span(doc, start={start}, end={end}, label='{label}')")
    E130 = ("You are running a narrow unicode build, which is incompatible "
            "with spacy >= 2.1.0. To fix this, reinstall Python and use a wide "
            "unicode build instead. You can also rebuild Python and set the "
            "--enable-unicode=ucs4 flag.")
    E131 = ("Cannot write the kb_id of an existing Span object because a Span "
            "is a read-only view of the underlying Token objects stored in "
            "the Doc. Instead, create a new Span object and specify the "
            "`kb_id` keyword argument, for example:\nfrom spacy.tokens "
            "import Span\nspan = Span(doc, start={start}, end={end}, "
            "label='{label}', kb_id='{kb_id}')")
    E132 = ("The vectors for entities and probabilities for alias '{alias}' "
            "should have equal length, but found {entities_length} and "
            "{probabilities_length} respectively.")
    E133 = ("The sum of prior probabilities for alias '{alias}' should not "
            "exceed 1, but found {sum}.")
    E134 = ("Entity '{entity}' is not defined in the Knowledge Base.")
    E135 = ("If you meant to replace a built-in component, use `create_pipe`: "
            "`nlp.replace_pipe('{name}', nlp.create_pipe('{name}'))`")
    E136 = ("This additional feature requires the jsonschema library to be "
            "installed:\npip install jsonschema")
    E137 = ("Expected 'dict' type, but got '{type}' from '{line}'. Make sure "
            "to provide a valid JSON object as input with either the `text` "
            "or `tokens` key. For more info, see the docs:\n"
            "https://spacy.io/api/cli#pretrain-jsonl")
    E138 = ("Invalid JSONL format for raw text '{text}'. Make sure the input "
            "includes either the `text` or `tokens` key. For more info, see "
            "the docs:\nhttps://spacy.io/api/cli#pretrain-jsonl")
    E139 = ("Knowledge Base for component '{name}' not initialized. Did you "
            "forget to call set_kb()?")
    E140 = ("The list of entities, prior probabilities and entity vectors "
            "should be of equal length.")
    E141 = ("Entity vectors should be of length {required} instead of the "
            "provided {found}.")
    E142 = ("Unsupported loss_function '{loss_func}'. Use either 'L2' or "
            "'cosine'.")
    E143 = ("Labels for component '{name}' not initialized. Did you forget to "
            "call add_label()?")
    E144 = ("Could not find parameter `{param}` when building the entity "
            "linker model.")
    E145 = ("Error reading `{param}` from input file.")
    E146 = ("Could not access `{path}`.")
    E147 = ("Unexpected error in the {method} functionality of the "
            "EntityLinker: {msg}. This is likely a bug in spaCy, so feel free "
            "to open an issue.")
    E148 = ("Expected {ents} KB identifiers but got {ids}. Make sure that "
            "each entity in `doc.ents` is assigned to a KB identifier.")
    E149 = ("Error deserializing model. Check that the config used to create "
            "the component matches the model being loaded.")
    E150 = ("The language of the `nlp` object and the `vocab` should be the "
            "same, but found '{nlp}' and '{vocab}' respectively.")
    E151 = ("Trying to call nlp.update without required annotation types. "
            "Expected top-level keys: {exp}. Got: {unexp}.")
    E152 = ("The attribute {attr} is not supported for token patterns. "
            "Please use the option validate=True with Matcher, PhraseMatcher, "
            "or EntityRuler for more details.")
    E153 = ("The value type {vtype} is not supported for token patterns. "
            "Please use the option validate=True with Matcher, PhraseMatcher, "
            "or EntityRuler for more details.")
    E154 = ("One of the attributes or values is not supported for token "
            "patterns. Please use the option validate=True with Matcher, "
            "PhraseMatcher, or EntityRuler for more details.")
    E155 = ("The pipeline needs to include a tagger in order to use "
            "Matcher or PhraseMatcher with the attributes POS, TAG, or LEMMA. "
            "Try using nlp() instead of nlp.make_doc() or list(nlp.pipe()) "
            "instead of list(nlp.tokenizer.pipe()).")
    E156 = ("The pipeline needs to include a parser in order to use "
            "Matcher or PhraseMatcher with the attribute DEP. Try using "
            "nlp() instead of nlp.make_doc() or list(nlp.pipe()) instead of "
            "list(nlp.tokenizer.pipe()).")
    E157 = ("Can't render negative values for dependency arc start or end. "
            "Make sure that you're passing in absolute token indices, not "
            "relative token offsets.\nstart: {start}, end: {end}, label: "
            "{label}, direction: {dir}")
    E158 = ("Can't add table '{name}' to lookups because it already exists.")
    E159 = ("Can't find table '{name}' in lookups. Available tables: {tables}")
    E160 = ("Can't find language data file: {path}")
    E161 = ("Found an internal inconsistency when predicting entity links. "
            "This is likely a bug in spaCy, so feel free to open an issue.")
    E162 = ("Cannot evaluate textcat model on data with different labels.\n"
            "Labels in model: {model_labels}\nLabels in evaluation "
            "data: {eval_labels}")
    E163 = ("cumsum was found to be unstable: its last element does not "
            "correspond to sum")
    E164 = ("x is neither increasing nor decreasing: {}.")
    E165 = ("Only one class present in y_true. ROC AUC score is not defined in "
            "that case.")
    E166 = ("Can only merge DocBins with the same pre-defined attributes.\n"
            "Current DocBin: {current}\nOther DocBin: {other}")
    E167 = ("Unknown morphological feature: '{feat}' ({feat_id}). This can "
            "happen if the tagger was trained with a different set of "
            "morphological features. If you're using a pretrained model, make "
            "sure that your models are up to date:\npython -m spacy validate")
    E168 = ("Unknown field: {field}")
    E169 = ("Can't find module: {module}")
    E170 = ("Cannot apply transition {name}: invalid for the current state.")
    E171 = ("Matcher.add received invalid on_match callback argument: expected "
            "callable or None, but got: {arg_type}")
    E172 = ("The Lemmatizer.load classmethod is deprecated. To create a "
            "Lemmatizer, initialize the class directly. See the docs for "
            "details: https://spacy.io/api/lemmatizer")
    E173 = ("As of v2.2, the Lemmatizer is initialized with an instance of "
            "Lookups containing the lemmatization tables. See the docs for "
            "details: https://spacy.io/api/lemmatizer#init")
    E174 = ("Architecture '{name}' not found in registry. Available "
            "names: {names}")
    E175 = ("Can't remove rule for unknown match pattern ID: {key}")
    E176 = ("Alias '{alias}' is not defined in the Knowledge Base.")
    E177 = ("Ill-formed IOB input detected: {tag}")
    E178 = ("Invalid pattern. Expected list of dicts but got: {pat}. Maybe you "
            "accidentally passed a single pattern to Matcher.add instead of a "
            "list of patterns? If you only want to add one pattern, make sure "
            "to wrap it in a list. For example: matcher.add('{key}', [pattern])")
    E179 = ("Invalid pattern. Expected a list of Doc objects but got a single "
            "Doc. If you only want to add one pattern, make sure to wrap it "
            "in a list. For example: matcher.add('{key}', [doc])")
    E180 = ("Span attributes can't be declared as required or assigned by "
            "components, since spans are only views of the Doc. Use Doc and "
            "Token attributes (or custom extension attributes) only and remove "
            "the following: {attrs}")
    E181 = ("Received invalid attributes for unkown object {obj}: {attrs}. "
            "Only Doc and Token attributes are supported.")
    E182 = ("Received invalid attribute declaration: {attr}\nDid you forget "
            "to define the attribute? For example: {attr}.???")
    E183 = ("Received invalid attribute declaration: {attr}\nOnly top-level "
            "attributes are supported, for example: {solution}")
    E184 = ("Only attributes without underscores are supported in component "
            "attribute declarations (because underscore and non-underscore "
            "attributes are connected anyways): {attr} -> {solution}")
    E185 = ("Received invalid attribute in component attribute declaration: "
            "{obj}.{attr}\nAttribute '{attr}' does not exist on {obj}.")
    E186 = ("'{tok_a}' and '{tok_b}' are different texts.")
    E187 = ("Only unicode strings are supported as labels.")
    E188 = ("Could not match the gold entity links to entities in the doc - "
            "make sure the gold EL data refers to valid results of the "
            "named entity recognizer in the `nlp` pipeline.")
    E189 = ("Each argument to `get_doc` should be of equal length.")
    E190 = ("Token head out of range in `Doc.from_array()` for token index "
            "'{index}' with value '{value}' (equivalent to relative head "
            "index: '{rel_head_index}'). The head indices should be relative "
            "to the current token index rather than absolute indices in the "
            "array.")
    E191 = ("Invalid head: the head token must be from the same doc as the "
            "token itself.")
    E192 = ("Unable to resize vectors in place with cupy.")
    E193 = ("Unable to resize vectors in place if the resized vector dimension "
            "({new_dim}) is not the same as the current vector dimension "
            "({curr_dim}).")
    E194 = ("Unable to aligned mismatched text '{text}' and words '{words}'.")
    E195 = ("Matcher can be called on {good} only, got {got}.")
    E196 = ("Refusing to write to token.is_sent_end. Sentence boundaries can "
            "only be fixed with token.is_sent_start.")
    E197 = ("Row out of bounds, unable to add row {row} for key {key}.")
    E198 = ("Unable to return {n} most similar vectors for the current vectors "
            "table, which contains {n_rows} vectors.")
    E199 = ("Unable to merge 0-length span at doc[{start}:{end}].")
    E200 = ("Specifying a base model with a pretrained component '{component}' "
            "can not be combined with adding a pretrained Tok2Vec layer.")
    E201 = ("Span index out of range.")


@add_codes
class TempErrors(object):
    T003 = ("Resizing pretrained Tagger models is not currently supported.")
    T004 = ("Currently parser depth is hard-coded to 1. Received: {value}.")
    T007 = ("Can't yet set {attr} from Span. Vote for this feature on the "
            "issue tracker: http://github.com/explosion/spaCy/issues")
    T008 = ("Bad configuration of Tagger. This is probably a bug within "
            "spaCy. We changed the name of an internal attribute for loading "
            "pretrained vectors, and the class has been passed the old name "
            "(pretrained_dims) but not the new name (pretrained_vectors).")


# fmt: on


class MatchPatternError(ValueError):
    def __init__(self, key, errors):
        """Custom error for validating match patterns.

        key (unicode): The name of the matcher rule.
        errors (dict): Validation errors (sequence of strings) mapped to pattern
            ID, i.e. the index of the added pattern.
        """
        msg = "Invalid token patterns for matcher rule '{}'\n".format(key)
        for pattern_idx, error_msgs in errors.items():
            pattern_errors = "\n".join(["- {}".format(e) for e in error_msgs])
            msg += "\nPattern {}:\n{}\n".format(pattern_idx, pattern_errors)
        ValueError.__init__(self, msg)


class AlignmentError(ValueError):
    pass



# encoding: utf8
from __future__ import unicode_literals


STOP_WORDS = set(
    """
a abans ací ah així això al aleshores algun alguna algunes alguns alhora allà allí allò
als altra altre altres amb ambdues ambdós anar ans apa aquell aquella aquelles aquells
aquest aquesta aquestes aquests aquí

baix bastant bé

cada cadascuna cadascunes cadascuns cadascú com consegueixo conseguim conseguir
consigueix consigueixen consigueixes contra

d'un d'una d'unes d'uns dalt de del dels des des de després dins dintre donat doncs durant

e eh el elles ells els em en encara ens entre era erem eren eres es esta estan estat
estava estaven estem esteu estic està estàvem estàveu et etc ets érem éreu és éssent

fa faig fan fas fem fer feu fi fins fora

gairebé

ha han has haver havia he hem heu hi ho

i igual iguals inclòs

ja jo

l'hi la les li li'n llarg llavors

m'he ma mal malgrat mateix mateixa mateixes mateixos me mentre meu meus meva
meves mode molt molta moltes molts mon mons més

n'he n'hi ne ni no nogensmenys només nosaltres nostra nostre nostres

o oh oi on

pas pel pels per per que perquè però poc poca pocs podem poden poder
podeu poques potser primer propi puc

qual quals quan quant que quelcom qui quin quina quines quins què

s'ha s'han sa sabem saben saber sabeu sap saps semblant semblants sense ser ses
seu seus seva seves si sobre sobretot soc solament sols som son sons sota sou sóc són

t'ha t'han t'he ta tal també tampoc tan tant tanta tantes te tene tenim tenir teniu
teu teus teva teves tinc ton tons tot tota totes tots

un una unes uns us últim ús

va vaig vam van vas veu vosaltres vostra vostre vostres

""".split()
)



# coding: utf8
from __future__ import unicode_literals

from spacy import displacy


def test_issue3531():
    """Test that displaCy renderer doesn't require "settings" key."""
    example_dep = {
        "words": [
            {"text": "But", "tag": "CCONJ"},
            {"text": "Google", "tag": "PROPN"},
            {"text": "is", "tag": "VERB"},
            {"text": "starting", "tag": "VERB"},
            {"text": "from", "tag": "ADP"},
            {"text": "behind.", "tag": "ADV"},
        ],
        "arcs": [
            {"start": 0, "end": 3, "label": "cc", "dir": "left"},
            {"start": 1, "end": 3, "label": "nsubj", "dir": "left"},
            {"start": 2, "end": 3, "label": "aux", "dir": "left"},
            {"start": 3, "end": 4, "label": "prep", "dir": "right"},
            {"start": 4, "end": 5, "label": "pcomp", "dir": "right"},
        ],
    }
    example_ent = {
        "text": "But Google is starting from behind.",
        "ents": [{"start": 4, "end": 10, "label": "ORG"}],
    }
    dep_html = displacy.render(example_dep, style="dep", manual=True)
    assert dep_html
    ent_html = displacy.render(example_ent, style="ent", manual=True)
    assert ent_html



# coding: utf8
from __future__ import unicode_literals

from ...attrs import LANG, NORM
from ..norm_exceptions import BASE_NORMS
from ...language import Language
from ...tokens import Doc
from .stop_words import STOP_WORDS
from ...util import add_lookups
from .lex_attrs import LEX_ATTRS


class VietnameseDefaults(Language.Defaults):
    lex_attr_getters = dict(Language.Defaults.lex_attr_getters)
    lex_attr_getters[LANG] = lambda text: "vi"  # for pickling
    lex_attr_getters[NORM] = add_lookups(
        Language.Defaults.lex_attr_getters[NORM], BASE_NORMS
    )
    lex_attr_getters.update(LEX_ATTRS)
    stop_words = STOP_WORDS
    use_pyvi = True


class Vietnamese(Language):
    lang = "vi"
    Defaults = VietnameseDefaults  # override defaults

    def make_doc(self, text):
        if self.Defaults.use_pyvi:
            try:
                from pyvi import ViTokenizer
            except ImportError:
                msg = (
                    "Pyvi not installed. Either set Vietnamese.use_pyvi = False, "
                    "or install it https://pypi.python.org/pypi/pyvi"
                )
                raise ImportError(msg)
            words, spaces = ViTokenizer.spacy_tokenize(text)
            return Doc(self.vocab, words=words, spaces=spaces)
        else:
            words = []
            spaces = []
            for token in self.tokenizer(text):
                words.extend(list(token.text))
                spaces.extend([False] * len(token.text))
                spaces[-1] = bool(token.whitespace_)
            return Doc(self.vocab, words=words, spaces=spaces)


__all__ = ["Vietnamese"]



# coding: utf8
from __future__ import unicode_literals

from ...symbols import POS, PUNCT, ADJ, CCONJ, NUM, DET, ADV, ADP, X, VERB
from ...symbols import NOUN, PROPN, PART, INTJ, SPACE, PRON, AUX, SCONJ


TAG_MAP = {
    "ADJ__Gender=Fem|Number=Plur": {POS: ADJ},
    "ADJ__Gender=Fem|Number=Plur|NumType=Ord": {POS: ADJ},
    "ADJ__Gender=Fem|Number=Sing": {POS: ADJ},
    "ADJ__Gender=Fem|Number=Sing|NumType=Ord": {POS: ADJ},
    "ADJ__Gender=Masc": {POS: ADJ},
    "ADJ__Gender=Masc|Number=Plur": {POS: ADJ},
    "ADJ__Gender=Masc|Number=Plur|NumType=Ord": {POS: ADJ},
    "ADJ__Gender=Masc|Number=Sing": {POS: ADJ},
    "ADJ__Gender=Masc|Number=Sing|NumType=Card": {POS: ADJ},
    "ADJ__Gender=Masc|Number=Sing|NumType=Ord": {POS: ADJ},
    "ADJ__NumType=Card": {POS: ADJ},
    "ADJ__NumType=Ord": {POS: ADJ},
    "ADJ__Number=Plur": {POS: ADJ},
    "ADJ__Number=Sing": {POS: ADJ},
    "ADJ__Number=Sing|NumType=Ord": {POS: ADJ},
    "ADJ___": {POS: ADJ},
    "ADP__Gender=Fem|Number=Plur|Person=3": {POS: ADP},
    "ADP__Gender=Masc|Number=Plur|Person=3": {POS: ADP},
    "ADP__Gender=Masc|Number=Sing|Person=3": {POS: ADP},
    "ADP___": {POS: ADP},
    "ADV__Polarity=Neg": {POS: ADV},
    "ADV__PronType=Int": {POS: ADV},
    "ADV___": {POS: ADV},
    "AUX__Gender=Fem|Number=Plur|Tense=Past|VerbForm=Part": {POS: AUX},
    "AUX__Gender=Fem|Number=Plur|Tense=Past|VerbForm=Part|Voice=Pass": {POS: AUX},
    "AUX__Gender=Fem|Number=Sing|Tense=Past|VerbForm=Part": {POS: AUX},
    "AUX__Gender=Fem|Number=Sing|Tense=Past|VerbForm=Part|Voice=Pass": {POS: AUX},
    "AUX__Gender=Masc|Number=Plur|Tense=Past|VerbForm=Part": {POS: AUX},
    "AUX__Gender=Masc|Number=Plur|Tense=Past|VerbForm=Part|Voice=Pass": {POS: AUX},
    "AUX__Gender=Masc|Number=Sing|Tense=Past|VerbForm=Part": {POS: AUX},
    "AUX__Gender=Masc|Number=Sing|Tense=Past|VerbForm=Part|Voice=Pass": {POS: AUX},
    "AUX__Mood=Cnd|Number=Plur|Person=1|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Cnd|Number=Plur|Person=2|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Cnd|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Cnd|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Cnd|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Imp|Number=Plur|Person=2|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Plur|Person=1|Tense=Fut|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Plur|Person=1|Tense=Imp|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Plur|Person=1|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Plur|Person=2|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Plur|Person=3|Tense=Fut|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Plur|Person=3|Tense=Imp|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Plur|Person=3|Tense=Past|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Sing|Person=1|Tense=Imp|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Sing|Person=2|Tense=Imp|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Sing|Person=3|Tense=Fut|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Sing|Person=3|Tense=Imp|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Sing|Person=3|Tense=Past|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Sub|Number=Plur|Person=1|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Sub|Number=Plur|Person=2|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Sub|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Sub|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Mood=Sub|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin": {POS: AUX},
    "AUX__Number=Sing|Tense=Past|VerbForm=Part|Voice=Pass": {POS: AUX},
    "AUX__Tense=Past|VerbForm=Part": {POS: AUX},
    "AUX__Tense=Pres|VerbForm=Part": {POS: AUX},
    "AUX__VerbForm=Inf": {POS: AUX},
    "CCONJ___": {POS: CCONJ},
    "DET__Definite=Def|Gender=Fem|Number=Sing|PronType=Art": {POS: DET},
    "DET__Definite=Def|Gender=Masc|Number=Sing|PronType=Art": {POS: DET},
    "DET__Definite=Def|Number=Plur|PronType=Art": {POS: DET},
    "DET__Definite=Def|Number=Sing|PronType=Art": {POS: DET},
    "DET__Definite=Ind|Gender=Fem|Number=Plur|PronType=Art": {POS: DET},
    "DET__Definite=Ind|Gender=Fem|Number=Sing|PronType=Art": {POS: DET},
    "DET__Definite=Ind|Gender=Masc|Number=Plur|PronType=Art": {POS: DET},
    "DET__Definite=Ind|Gender=Masc|Number=Sing|PronType=Art": {POS: DET},
    "DET__Definite=Ind|Number=Plur|PronType=Art": {POS: DET},
    "DET__Definite=Ind|Number=Sing|PronType=Art": {POS: DET},
    "DET__Gender=Fem|Number=Plur": {POS: DET},
    "DET__Gender=Fem|Number=Plur|PronType=Int": {POS: DET},
    "DET__Gender=Fem|Number=Sing": {POS: DET},
    "DET__Gender=Fem|Number=Sing|Poss=Yes": {POS: DET},
    "DET__Gender=Fem|Number=Sing|PronType=Dem": {POS: DET},
    "DET__Gender=Fem|Number=Sing|PronType=Int": {POS: DET},
    "DET__Gender=Masc|Number=Plur": {POS: DET},
    "DET__Gender=Masc|Number=Sing": {POS: DET},
    "DET__Gender=Masc|Number=Sing|PronType=Dem": {POS: DET},
    "DET__Gender=Masc|Number=Sing|PronType=Int": {POS: DET},
    "DET__Number=Plur": {POS: DET},
    "DET__Number=Plur|Poss=Yes": {POS: DET},
    "DET__Number=Plur|PronType=Dem": {POS: DET},
    "DET__Number=Sing": {POS: DET},
    "DET__Number=Sing|Poss=Yes": {POS: DET},
    "DET___": {POS: DET},
    "INTJ___": {POS: INTJ},
    "NOUN__Gender=Fem": {POS: NOUN},
    "NOUN__Gender=Fem|Number=Plur": {POS: NOUN},
    "NOUN__Gender=Fem|Number=Sing": {POS: NOUN},
    "NOUN__Gender=Masc": {POS: NOUN},
    "NOUN__Gender=Masc|Number=Plur": {POS: NOUN},
    "NOUN__Gender=Masc|Number=Plur|NumType=Card": {POS: NOUN},
    "NOUN__Gender=Masc|Number=Sing": {POS: NOUN},
    "NOUN__Gender=Masc|Number=Sing|NumType=Card": {POS: NOUN},
    "NOUN__NumType=Card": {POS: NOUN},
    "NOUN__Number=Plur": {POS: NOUN},
    "NOUN__Number=Sing": {POS: NOUN},
    "NOUN___": {POS: NOUN},
    "NUM__Gender=Masc|Number=Plur|NumType=Card": {POS: NUM},
    "NUM__NumType=Card": {POS: NUM},
    "PART___": {POS: PART},
    "PRON__Gender=Fem|Number=Plur": {POS: PRON},
    "PRON__Gender=Fem|Number=Plur|Person=3": {POS: PRON},
    "PRON__Gender=Fem|Number=Plur|Person=3|PronType=Prs": {POS: PRON},
    "PRON__Gender=Fem|Number=Plur|Person=3|PronType=Rel": {POS: PRON},
    "PRON__Gender=Fem|Number=Plur|PronType=Dem": {POS: PRON},
    "PRON__Gender=Fem|Number=Plur|PronType=Rel": {POS: PRON},
    "PRON__Gender=Fem|Number=Sing|Person=3": {POS: PRON},
    "PRON__Gender=Fem|Number=Sing|Person=3|PronType=Prs": {POS: PRON},
    "PRON__Gender=Fem|Number=Sing|PronType=Dem": {POS: PRON},
    "PRON__Gender=Fem|Number=Sing|PronType=Rel": {POS: PRON},
    "PRON__Gender=Fem|PronType=Rel": {POS: PRON},
    "PRON__Gender=Masc|Number=Plur": {POS: PRON},
    "PRON__Gender=Masc|Number=Plur|Person=3": {POS: PRON},
    "PRON__Gender=Masc|Number=Plur|Person=3|PronType=Prs": {POS: PRON},
    "PRON__Gender=Masc|Number=Plur|Person=3|PronType=Rel": {POS: PRON},
    "PRON__Gender=Masc|Number=Plur|PronType=Dem": {POS: PRON},
    "PRON__Gender=Masc|Number=Plur|PronType=Rel": {POS: PRON},
    "PRON__Gender=Masc|Number=Sing": {POS: PRON},
    "PRON__Gender=Masc|Number=Sing|Person=3": {POS: PRON},
    "PRON__Gender=Masc|Number=Sing|Person=3|PronType=Dem": {POS: PRON},
    "PRON__Gender=Masc|Number=Sing|Person=3|PronType=Prs": {POS: PRON},
    "PRON__Gender=Masc|Number=Sing|PronType=Dem": {POS: PRON},
    "PRON__Gender=Masc|Number=Sing|PronType=Rel": {POS: PRON},
    "PRON__Gender=Masc|PronType=Rel": {POS: PRON},
    "PRON__NumType=Card|PronType=Rel": {POS: PRON},
    "PRON__Number=Plur|Person=1": {POS: PRON},
    "PRON__Number=Plur|Person=1|PronType=Prs": {POS: PRON},
    "PRON__Number=Plur|Person=1|Reflex=Yes": {POS: PRON},
    "PRON__Number=Plur|Person=2": {POS: PRON},
    "PRON__Number=Plur|Person=2|PronType=Prs": {POS: PRON},
    "PRON__Number=Plur|Person=2|Reflex=Yes": {POS: PRON},
    "PRON__Number=Plur|Person=3": {POS: PRON},
    "PRON__Number=Plur|PronType=Rel": {POS: PRON},
    "PRON__Number=Sing|Person=1": {POS: PRON},
    "PRON__Number=Sing|Person=1|PronType=Prs": {POS: PRON},
    "PRON__Number=Sing|Person=1|Reflex=Yes": {POS: PRON},
    "PRON__Number=Sing|Person=2|PronType=Prs": {POS: PRON},
    "PRON__Number=Sing|Person=3": {POS: PRON},
    "PRON__Number=Sing|PronType=Dem": {POS: PRON},
    "PRON__Number=Sing|PronType=Rel": {POS: PRON},
    "PRON__Person=3": {POS: PRON},
    "PRON__Person=3|Reflex=Yes": {POS: PRON},
    "PRON__PronType=Int": {POS: PRON},
    "PRON__PronType=Rel": {POS: PRON},
    "PRON___": {POS: PRON},
    "PROPN__Gender=Fem|Number=Plur": {POS: PROPN},
    "PROPN__Gender=Fem|Number=Sing": {POS: PROPN},
    "PROPN__Gender=Masc": {POS: PROPN},
    "PROPN__Gender=Masc|Number=Plur": {POS: PROPN},
    "PROPN__Gender=Masc|Number=Sing": {POS: PROPN},
    "PROPN__Number=Plur": {POS: PROPN},
    "PROPN__Number=Sing": {POS: PROPN},
    "PROPN___": {POS: PROPN},
    "PUNCT___": {POS: PUNCT},
    "SCONJ___": {POS: SCONJ},
    "VERB__Gender=Fem|Number=Plur|Tense=Past|VerbForm=Part": {POS: VERB},
    "VERB__Gender=Fem|Number=Plur|Tense=Past|VerbForm=Part|Voice=Pass": {POS: VERB},
    "VERB__Gender=Fem|Number=Sing|Tense=Past|VerbForm=Part": {POS: VERB},
    "VERB__Gender=Fem|Number=Sing|Tense=Past|VerbForm=Part|Voice=Pass": {POS: VERB},
    "VERB__Gender=Masc|Number=Plur|Tense=Past|VerbForm=Part": {POS: VERB},
    "VERB__Gender=Masc|Number=Plur|Tense=Past|VerbForm=Part|Voice=Pass": {POS: VERB},
    "VERB__Gender=Masc|Number=Sing|Tense=Past|VerbForm=Part": {POS: VERB},
    "VERB__Gender=Masc|Number=Sing|Tense=Past|VerbForm=Part|Voice=Pass": {POS: VERB},
    "VERB__Gender=Masc|Tense=Past|VerbForm=Part": {POS: VERB},
    "VERB__Gender=Masc|Tense=Past|VerbForm=Part|Voice=Pass": {POS: VERB},
    "VERB__Mood=Cnd|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Cnd|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Cnd|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Imp|Number=Plur|Person=1|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Imp|Number=Plur|Person=2|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Imp|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Plur|Person=1|Tense=Fut|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Plur|Person=1|Tense=Imp|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Plur|Person=1|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Plur|Person=2|Tense=Fut|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Plur|Person=2|Tense=Imp|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Plur|Person=2|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Plur|Person=3|Tense=Fut|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Plur|Person=3|Tense=Imp|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Plur|Person=3|Tense=Past|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Sing|Person=1|Tense=Fut|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Sing|Person=1|Tense=Imp|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Sing|Person=3|Tense=Fut|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Sing|Person=3|Tense=Imp|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Sing|Person=3|Tense=Past|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Person=3|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|Person=3|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Ind|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Sub|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Sub|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Sub|Number=Sing|Person=3|Tense=Past|VerbForm=Fin": {POS: VERB},
    "VERB__Mood=Sub|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin": {POS: VERB},
    "VERB__Number=Plur|Tense=Past|VerbForm=Part": {POS: VERB},
    "VERB__Number=Plur|Tense=Past|VerbForm=Part|Voice=Pass": {POS: VERB},
    "VERB__Number=Sing|Tense=Past|VerbForm=Part": {POS: VERB},
    "VERB__Number=Sing|Tense=Past|VerbForm=Part|Voice=Pass": {POS: VERB},
    "VERB__Tense=Past|VerbForm=Part": {POS: VERB},
    "VERB__Tense=Past|VerbForm=Part|Voice=Pass": {POS: VERB},
    "VERB__Tense=Pres|VerbForm=Part": {POS: VERB},
    "VERB__VerbForm=Inf": {POS: VERB},
    "VERB__VerbForm=Part": {POS: VERB},
    "X___": {POS: X},
    "_SP": {POS: SPACE},
}



from __future__ import unicode_literals

from ...symbols import POS, ADJ, ADP, ADV, INTJ, NOUN, NUM, PART
from ...symbols import PRON, PROPN, PUNCT, SYM, VERB, X, CCONJ, SCONJ, DET, AUX

TAG_MAP = {
    "Afcfson": {
        
        "Degree": "Cmp",
        "Gender": "Fem",
        "Number": "Sing",
        POS: ADJ,
    },
    "Afcfsrn": {
        
        "Degree": "Cmp",
        "Gender": "Fem",
        "Number": "Sing",
        POS: ADJ,
    },
    "Afp": {"Degree": "Pos", POS: ADJ},
    "Afp-p-n": {"Degree": "Pos", "Number": "Plur", POS: ADJ},
    "Afp-p-ny": {"Degree": "Pos", "Number": "Plur", POS: ADJ},
    "Afp-poy": { "Degree": "Pos", "Number": "Plur", POS: ADJ},
    "Afpf--n": {"Degree": "Pos", "Gender": "Fem", POS: ADJ},
    "Afpfp-n": {"Degree": "Pos", "Gender": "Fem", "Number": "Plur", POS: ADJ},
    "Afpfpoy": {
        
        "Degree": "Pos",
        "Gender": "Fem",
        "Number": "Plur",
        POS: ADJ,
    },
    "Afpfpry": {
        
        "Degree": "Pos",
        "Gender": "Fem",
        "Number": "Plur",
        POS: ADJ,
    },
    "Afpfson": {
        
        "Degree": "Pos",
        "Gender": "Fem",
        "Number": "Sing",
        POS: ADJ,
    },
    "Afpfsoy": {
        
        "Degree": "Pos",
        "Gender": "Fem",
        "Number": "Sing",
        POS: ADJ,
    },
    "Afpfsrn": {
        
        "Degree": "Pos",
        "Gender": "Fem",
        "Number": "Sing",
        POS: ADJ,
    },
    "Afpfsry": {
        
        "Degree": "Pos",
        "Gender": "Fem",
        "Number": "Sing",
        POS: ADJ,
    },
    "Afpmp-n": {"Degree": "Pos", "Gender": "Masc", "Number": "Plur", POS: ADJ},
    "Afpmpoy": {
        
        "Degree": "Pos",
        "Gender": "Masc",
        "Number": "Plur",
        POS: ADJ,
    },
    "Afpmpry": {
        
        "Degree": "Pos",
        "Gender": "Masc",
        "Number": "Plur",
        POS: ADJ,
    },
    "Afpms-n": {"Degree": "Pos", "Gender": "Masc", "Number": "Sing", POS: ADJ},
    "Afpmsoy": {
        
        "Degree": "Pos",
        "Gender": "Masc",
        "Number": "Sing",
        POS: ADJ,
    },
    "Afpmsry": {
        
        "Degree": "Pos",
        "Gender": "Masc",
        "Number": "Sing",
        POS: ADJ,
    },
    "COLON": {POS: PUNCT},
    "COMMA": {POS: PUNCT},
    "Ccssp": {POS: CCONJ, "Polarity": "Pos"},
    "Crssp": {POS: CCONJ, "Polarity": "Pos"},
    "Csssp": {POS: SCONJ, "Polarity": "Pos"},
    "Cssspy": {POS: SCONJ, "Polarity": "Pos"},
    "DASH": {POS: PUNCT},
    "DBLQ": {POS: PUNCT},
    "Dd3-po---e": {
        
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "PronType": "Dem",
    },
    "Dd3fpr": {
        
        "Gender": "Fem",
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "PronType": "Dem",
    },
    "Dd3fpr---e": {
        
        "Gender": "Fem",
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "PronType": "Dem",
    },
    "Dd3fso---e": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Dem",
    },
    "Dd3fso---o": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Dem",
    },
    "Dd3fsr": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Dem",
    },
    "Dd3fsr---e": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Dem",
    },
    "Dd3fsr---o": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Dem",
    },
    "Dd3mpo": {
        
        "Gender": "Masc",
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "PronType": "Dem",
    },
    "Dd3mpr---e": {
        
        "Gender": "Masc",
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "PronType": "Dem",
    },
    "Dd3mso---e": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Dem",
    },
    "Dd3msr---e": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Dem",
    },
    "Dd3msr---o": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Dem",
    },
    "Dh3fsr": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        
    },
    "Dh3mp": {
        "Gender": "Masc",
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        
    },
    "Dh3ms": {
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        
    },
    "Di3": {POS: DET, "Person": "three", "PronType": "Ind"},
    "Di3--r---e": { POS: DET, "Person": "three", "PronType": "Ind"},
    "Di3-po": {
        
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3-po---e": {
        
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3-sr": {
        
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3-sr---e": {
        
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3fp": {
        "Gender": "Fem",
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3fpr": {
        
        "Gender": "Fem",
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3fpr---e": {
        
        "Gender": "Fem",
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3fso---e": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3fsr": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3fsr---e": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3mp": {
        "Gender": "Masc",
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3mpr": {
        
        "Gender": "Masc",
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3mpr---e": {
        
        "Gender": "Masc",
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3ms": {
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3ms----e": {
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3mso---e": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3msr": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Di3msr---e": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Ind",
    },
    "Ds1fp-s": {
        "Gender": "Fem",
        "Number": "Plur",
        POS: DET,
        "Person": "one",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Ds1fsos": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "one",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Ds1fsrp": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "one",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Ds1fsrs": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "one",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Ds1ms-p": {
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "one",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Ds1ms-s": {
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "one",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Ds2---s": {POS: DET, "Person": "two", "Poss": "Yes", "PronType": "Prs"},
    "Ds2fsrs": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "two",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Ds3---p": {POS: DET, "Person": "three", "Poss": "Yes", "PronType": "Prs"},
    "Ds3---s": {POS: DET, "Person": "three", "Poss": "Yes", "PronType": "Prs"},
    "Ds3fp-s": {
        "Gender": "Fem",
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Ds3fsos": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Ds3fsrs": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Ds3ms-s": {
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Dw3--r---e": { POS: DET, "Person": "three"},
    "Dw3fpr": {
        
        "Gender": "Fem",
        "Number": "Plur",
        POS: DET,
        "Person": "three",
        
    },
    "Dw3mso---e": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        
    },
    "Dz3fsr---e": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Neg",
    },
    "Dz3msr---e": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Person": "three",
        "PronType": "Neg",
    },
    "EQUAL": {POS: SYM},
    "EXCL": {POS: PUNCT},
    "GT": {POS: SYM},
    "I": {POS: INTJ},
    "LPAR": {POS: PUNCT},
    "Mc": {"NumType": "Card", POS: NUM},
    "Mc-p-d": {"NumForm": "Digit", "NumType": "Card", "Number": "Plur", POS: NUM},
    "Mc-p-l": {"NumForm": "Word", "NumType": "Card", "Number": "Plur", POS: NUM},
    "Mcfp-l": {
        "Gender": "Fem",
        "NumForm": "Word",
        "NumType": "Card",
        "Number": "Plur",
        POS: NUM,
    },
    "Mcfp-ln": {
        "Gender": "Fem",
        "NumForm": "Word",
        "NumType": "Card",
        "Number": "Plur",
        POS: NUM,
    },
    "Mcfsrln": {
        
        "Gender": "Fem",
        "NumForm": "Word",
        "NumType": "Card",
        "Number": "Sing",
        POS: NUM,
    },
    "Mcmp-l": {
        "Gender": "Masc",
        "NumForm": "Word",
        "NumType": "Card",
        "Number": "Plur",
        POS: NUM,
    },
    "Mcmsrl": {
        
        "Gender": "Masc",
        "NumForm": "Word",
        "NumType": "Card",
        "Number": "Sing",
        POS: NUM,
    },
    "Mffprln": {
        
        "Gender": "Fem",
        "NumForm": "Word",
        "NumType": "Card",
        "Number": "Plur",
        POS: NUM,
    },
    "Mlfpo": {
        
        "Gender": "Fem",
        "NumType": "Card",
        "Number": "Plur",
        POS: NUM,
        "PronType": "Tot",
    },
    "Mlfpr": {
        
        "Gender": "Fem",
        "NumType": "Card",
        "Number": "Plur",
        POS: NUM,
        "PronType": "Tot",
    },
    "Mlmpr": {
        
        "Gender": "Masc",
        "NumType": "Card",
        "Number": "Plur",
        POS: NUM,
        "PronType": "Tot",
    },
    "Mo---l": {"NumForm": "Word", "NumType": "Ord", POS: NUM},
    "Mo-s-r": {"NumForm": "Roman", "NumType": "Ord", "Number": "Sing", POS: NUM},
    "Mofp-ln": {
        "Gender": "Fem",
        "NumForm": "Word",
        "NumType": "Ord",
        "Number": "Plur",
        POS: NUM,
    },
    "Mofprly": {
        
        "Gender": "Fem",
        "NumForm": "Word",
        "NumType": "Ord",
        "Number": "Plur",
        POS: NUM,
    },
    "Mofs-l": {
        "Gender": "Fem",
        "NumForm": "Word",
        "NumType": "Ord",
        "Number": "Sing",
        POS: NUM,
    },
    "Mofsrln": {
        
        "Gender": "Fem",
        "NumForm": "Word",
        "NumType": "Ord",
        "Number": "Sing",
        POS: NUM,
    },
    "Mofsrly": {
        
        "Gender": "Fem",
        "NumForm": "Word",
        "NumType": "Ord",
        "Number": "Sing",
        POS: NUM,
    },
    "Momprly": {
        
        "Gender": "Masc",
        "NumForm": "Word",
        "NumType": "Ord",
        "Number": "Plur",
        POS: NUM,
    },
    "Moms-l": {
        "Gender": "Masc",
        "NumForm": "Word",
        "NumType": "Ord",
        "Number": "Sing",
        POS: NUM,
    },
    "Moms-ln": {
        "Gender": "Masc",
        "NumForm": "Word",
        "NumType": "Ord",
        "Number": "Sing",
        POS: NUM,
    },
    "Momsoly": {
        
        "Gender": "Masc",
        "NumForm": "Word",
        "NumType": "Ord",
        "Number": "Sing",
        POS: NUM,
    },
    "Momsrly": {
        
        "Gender": "Masc",
        "NumForm": "Word",
        "NumType": "Ord",
        "Number": "Sing",
        POS: NUM,
    },
    "Nc": {POS: NOUN},
    "Ncf--n": {"Gender": "Fem", POS: NOUN},
    "Ncfp-n": {"Gender": "Fem", "Number": "Plur", POS: NOUN},
    "Ncfpoy": { "Gender": "Fem", "Number": "Plur", POS: NOUN},
    "Ncfpry": { "Gender": "Fem", "Number": "Plur", POS: NOUN},
    "Ncfson": { "Gender": "Fem", "Number": "Sing", POS: NOUN},
    "Ncfsoy": { "Gender": "Fem", "Number": "Sing", POS: NOUN},
    "Ncfsrn": { "Gender": "Fem", "Number": "Sing", POS: NOUN},
    "Ncfsry": { "Gender": "Fem", "Number": "Sing", POS: NOUN},
    "Ncm--n": {"Gender": "Masc", POS: NOUN},
    "Ncmp-n": {"Gender": "Masc", "Number": "Plur", POS: NOUN},
    "Ncmpoy": { "Gender": "Masc", "Number": "Plur", POS: NOUN},
    "Ncmpry": { "Gender": "Masc", "Number": "Plur", POS: NOUN},
    "Ncms-n": {"Gender": "Masc", "Number": "Sing", POS: NOUN},
    "Ncms-ny": {"Gender": "Masc", "Number": "Sing", POS: NOUN},
    "Ncmsoy": { "Gender": "Masc", "Number": "Sing", POS: NOUN},
    "Ncmsrn": { "Gender": "Masc", "Number": "Sing", POS: NOUN},
    "Ncmsry": { "Gender": "Masc", "Number": "Sing", POS: NOUN},
    "Np": {POS: PROPN},
    "Npfsoy": { "Gender": "Fem", "Number": "Sing", POS: PROPN},
    "Npfsry": { "Gender": "Fem", "Number": "Sing", POS: PROPN},
    "Npmsoy": { "Gender": "Masc", "Number": "Sing", POS: PROPN},
    "Npmsry": { "Gender": "Masc", "Number": "Sing", POS: PROPN},
    "PERCENT": {POS: SYM},
    "PERIOD": {POS: PUNCT},
    "PLUSMINUS": {POS: SYM},
    "Pd3-po": {
        
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Dem",
    },
    "Pd3fpr": {
        
        "Gender": "Fem",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Dem",
    },
    "Pd3fso": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Dem",
    },
    "Pd3fsr": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Dem",
    },
    "Pd3mpr": {
        
        "Gender": "Masc",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Dem",
    },
    "Pd3mso": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Dem",
    },
    "Pd3msr": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Dem",
    },
    "Pi3--r": { POS: PRON, "Person": "three", "PronType": "Ind"},
    "Pi3-po": {
        
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Ind",
    },
    "Pi3-so": {
        
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Ind",
    },
    "Pi3-sr": {
        
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Ind",
    },
    "Pi3fpr": {
        
        "Gender": "Fem",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Ind",
    },
    "Pi3fso": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Ind",
    },
    "Pi3fsr": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Ind",
    },
    "Pi3mpr": {
        
        "Gender": "Masc",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Ind",
    },
    "Pi3msr": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Ind",
    },
    "Pi3msr--y": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Ind",
        
    },
    "Pp1-pa--------w": {
        "Case": "Acc",
        "Number": "Plur",
        POS: PRON,
        "Person": "one",
        "PronType": "Prs",
    },
    "Pp1-pa--y-----w": {
        "Case": "Acc",
        "Number": "Plur",
        POS: PRON,
        "Person": "one",
        "PronType": "Prs",
        
    },
    "Pp1-pd--------w": {
        "Case": "Dat",
        "Number": "Plur",
        POS: PRON,
        "Person": "one",
        "PronType": "Prs",
    },
    "Pp1-pr--------s": {
        
        "Number": "Plur",
        POS: PRON,
        "Person": "one",
        "PronType": "Prs",
    },
    "Pp1-sa--------s": {
        "Case": "Acc",
        "Number": "Sing",
        POS: PRON,
        "Person": "one",
        "PronType": "Prs",
    },
    "Pp1-sa--------w": {
        "Case": "Acc",
        "Number": "Sing",
        POS: PRON,
        "Person": "one",
        "PronType": "Prs",
    },
    "Pp1-sa--y-----w": {
        "Case": "Acc",
        "Number": "Sing",
        POS: PRON,
        "Person": "one",
        "PronType": "Prs",
        
    },
    "Pp1-sd--------w": {
        "Case": "Dat",
        "Number": "Sing",
        POS: PRON,
        "Person": "one",
        "PronType": "Prs",
    },
    "Pp1-sd--y-----w": {
        "Case": "Dat",
        "Number": "Sing",
        POS: PRON,
        "Person": "one",
        "PronType": "Prs",
        
    },
    "Pp1-sn--------s": {
        "Case": "Nom",
        "Number": "Sing",
        POS: PRON,
        "Person": "one",
        "PronType": "Prs",
    },
    "Pp2-----------s": {POS: PRON, "Person": "two", "PronType": "Prs"},
    "Pp2-pa--------w": {
        "Case": "Acc",
        "Number": "Plur",
        POS: PRON,
        "Person": "two",
        "PronType": "Prs",
    },
    "Pp2-pa--y-----w": {
        "Case": "Acc",
        "Number": "Plur",
        POS: PRON,
        "Person": "two",
        "PronType": "Prs",
        
    },
    "Pp2-pd--------w": {
        "Case": "Dat",
        "Number": "Plur",
        POS: PRON,
        "Person": "two",
        "PronType": "Prs",
    },
    "Pp2-pr--------s": {
        
        "Number": "Plur",
        POS: PRON,
        "Person": "two",
        "PronType": "Prs",
    },
    "Pp2-sa--------s": {
        "Case": "Acc",
        "Number": "Sing",
        POS: PRON,
        "Person": "two",
        "PronType": "Prs",
    },
    "Pp2-sa--------w": {
        "Case": "Acc",
        "Number": "Sing",
        POS: PRON,
        "Person": "two",
        "PronType": "Prs",
    },
    "Pp2-sa--y-----w": {
        "Case": "Acc",
        "Number": "Sing",
        POS: PRON,
        "Person": "two",
        "PronType": "Prs",
        
    },
    "Pp2-sd--y-----w": {
        "Case": "Dat",
        "Number": "Sing",
        POS: PRON,
        "Person": "two",
        "PronType": "Prs",
        
    },
    "Pp2-sn--------s": {
        "Case": "Nom",
        "Number": "Sing",
        POS: PRON,
        "Person": "two",
        "PronType": "Prs",
    },
    "Pp3-pd--------w": {
        "Case": "Dat",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
    },
    "Pp3-pd--y-----w": {
        "Case": "Dat",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
        
    },
    "Pp3-po--------s": {
        
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
    },
    "Pp3-sd--------w": {
        "Case": "Dat",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
    },
    "Pp3-sd--y-----w": {
        "Case": "Dat",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
        
    },
    "Pp3fpa--------w": {
        "Case": "Acc",
        "Gender": "Fem",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
    },
    "Pp3fpa--y-----w": {
        "Case": "Acc",
        "Gender": "Fem",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
        
    },
    "Pp3fpr--------s": {
        
        "Gender": "Fem",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
    },
    "Pp3fsa--------w": {
        "Case": "Acc",
        "Gender": "Fem",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
    },
    "Pp3fsa--y-----w": {
        "Case": "Acc",
        "Gender": "Fem",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
        
    },
    "Pp3fsr--------s": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
    },
    "Pp3mpa--------w": {
        "Case": "Acc",
        "Gender": "Masc",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
    },
    "Pp3mpa--y-----w": {
        "Case": "Acc",
        "Gender": "Masc",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
        
    },
    "Pp3mpr--------s": {
        
        "Gender": "Masc",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
    },
    "Pp3msa--------w": {
        "Case": "Acc",
        "Gender": "Masc",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
    },
    "Pp3msa--y-----w": {
        "Case": "Acc",
        "Gender": "Masc",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
        
    },
    "Pp3mso--------s": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
    },
    "Pp3msr--------s": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
    },
    "Ps1mp-s": {
        "Gender": "Masc",
        "Number": "Plur",
        POS: PRON,
        "Person": "one",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Ps3---p": {POS: PRON, "Person": "three", "Poss": "Yes", "PronType": "Prs"},
    "Ps3---s": {POS: PRON, "Person": "three", "Poss": "Yes", "PronType": "Prs"},
    "Ps3fp-s": {
        "Gender": "Fem",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Pw3--r": { POS: PRON, "Person": "three"},
    "Pw3-po": {
        
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        
    },
    "Pw3fso": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        
    },
    "Pw3mpr": {
        
        "Gender": "Masc",
        "Number": "Plur",
        POS: PRON,
        "Person": "three",
        
    },
    "Px3--a--------s": {
        "Case": "Acc",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
        "Reflex": "Yes",
    },
    "Px3--a--------w": {
        "Case": "Acc",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
        "Reflex": "Yes",
    },
    "Px3--a--y-----w": {
        "Case": "Acc",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
        "Reflex": "Yes",
        
    },
    "Px3--d--------w": {
        "Case": "Dat",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
        "Reflex": "Yes",
    },
    "Px3--d--y-----w": {
        "Case": "Dat",
        POS: PRON,
        "Person": "three",
        "PronType": "Prs",
        "Reflex": "Yes",
        
    },
    "Pz3-sr": {
        
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Neg",
    },
    "Pz3msr": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: PRON,
        "Person": "three",
        "PronType": "Neg",
    },
    "QUEST": {POS: PUNCT},
    "QUOT": {POS: PUNCT},
    "Qn": {POS: PART, "PartType": "Inf"},
    "Qs": {"Mood": "Sub", POS: PART},
    "Qs-y": {"Mood": "Sub", POS: PART},
    "Qz": {POS: PART, "Polarity": "Neg"},
    "Qz-y": {POS: PART, "Polarity": "Neg"},
    "RPAR": {POS: PUNCT},
    "Rc": {POS: ADV},
    "Rgp": {"Degree": "Pos", POS: ADV},
    "Rgpy": {"Degree": "Pos", POS: ADV},
    "Rgs": {"Degree": "Sup", POS: ADV},
    "Rp": {POS: ADV},
    "Rw": {POS: ADV},
    "Rz": {POS: ADV, "PronType": "Neg"},
    "SCOLON": {"AdpType": "Prep", POS: PUNCT},
    "SLASH": {"AdpType": "Prep", POS: SYM},
    "Spsa": {"AdpType": "Prep", "Case": "Acc", POS: ADP},
    "Spsay": {"AdpType": "Prep", "Case": "Acc", POS: ADP},
    "Spsd": {"AdpType": "Prep", "Case": "Dat", POS: ADP},
    "Spsg": {"AdpType": "Prep", "Case": "Gen", POS: ADP},
    "Spsgy": {"AdpType": "Prep", "Case": "Gen", POS: ADP},
    "Td-po": { "Number": "Plur", POS: DET, "PronType": "Dem"},
    "Tdfpr": {
        
        "Gender": "Fem",
        "Number": "Plur",
        POS: DET,
        "PronType": "Dem",
    },
    "Tdfso": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "PronType": "Dem",
    },
    "Tdfsr": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "PronType": "Dem",
    },
    "Tdmpr": {
        
        "Gender": "Masc",
        "Number": "Plur",
        POS: DET,
        "PronType": "Dem",
    },
    "Tdmso": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "PronType": "Dem",
    },
    "Tdmsr": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "PronType": "Dem",
    },
    "Tf-so": { "Number": "Sing", POS: DET, "PronType": "Art"},
    "Tffs-y": {
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "PronType": "Art",
        
    },
    "Tfms-y": {
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "PronType": "Art",
        
    },
    "Tfmsoy": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "PronType": "Art",
        
    },
    "Tfmsry": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "PronType": "Art",
        
    },
    "Ti-po": { "Number": "Plur", POS: DET, "PronType": "Ind"},
    "Tifp-y": {
        "Gender": "Fem",
        "Number": "Plur",
        POS: DET,
        "PronType": "Ind",
        
    },
    "Tifso": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "PronType": "Ind",
    },
    "Tifsr": {
        
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "PronType": "Ind",
    },
    "Timso": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "PronType": "Ind",
    },
    "Timsr": {
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "PronType": "Ind",
    },
    "Tsfp": {
        "Gender": "Fem",
        "Number": "Plur",
        POS: DET,
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Tsfs": {
        "Gender": "Fem",
        "Number": "Sing",
        POS: DET,
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Tsmp": {
        "Gender": "Masc",
        "Number": "Plur",
        POS: DET,
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Tsms": {
        "Gender": "Masc",
        "Number": "Sing",
        POS: DET,
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "Va--1": {POS: AUX, "Person": "one"},
    "Va--1p": {"Number": "Plur", POS: AUX, "Person": "one"},
    "Va--1s": {"Number": "Sing", POS: AUX, "Person": "one"},
    "Va--2p": {"Number": "Plur", POS: AUX, "Person": "two"},
    "Va--2s": {"Number": "Sing", POS: AUX, "Person": "two"},
    "Va--3": {POS: AUX, "Person": "three"},
    "Va--3-----y": {POS: AUX, "Person": "three"},
    "Va--3p": {"Number": "Plur", POS: AUX, "Person": "three"},
    "Va--3p----y": {"Number": "Plur", POS: AUX, "Person": "three"},
    "Va--3s": {"Number": "Sing", POS: AUX, "Person": "three"},
    "Va--3s----y": {"Number": "Sing", POS: AUX, "Person": "three"},
    "Vag": {POS: AUX, "VerbForm": "Ger"},
    "Vaii3p": {
        "Mood": "Ind",
        "Number": "Plur",
        POS: AUX,
        "Person": "three",
        "Tense": "Imp",
        "VerbForm": "Fin",
    },
    "Vaii3s": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: AUX,
        "Person": "three",
        "Tense": "Imp",
        "VerbForm": "Fin",
    },
    "Vail3s": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: AUX,
        "Person": "three",
        
        "VerbForm": "Fin",
    },
    "Vaip1s": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: AUX,
        "Person": "one",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "Vaip2s": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: AUX,
        "Person": "two",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "Vaip3p": {
        "Mood": "Ind",
        "Number": "Plur",
        POS: AUX,
        "Person": "three",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "Vaip3s": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: AUX,
        "Person": "three",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "Vanp": {POS: AUX, "Tense": "Pres", "VerbForm": "Inf"},
    "Vap--sm": {"Gender": "Masc", "Number": "Sing", POS: AUX, "VerbForm": "Part"},
    "Vasp3": {
        "Mood": "Sub",
        POS: AUX,
        "Person": "three",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "Vmg": {POS: VERB, "VerbForm": "Ger"},
    "Vmg-------y": {POS: VERB,  "VerbForm": "Ger"},
    "Vmii1": {
        "Mood": "Ind",
        POS: VERB,
        "Person": "one",
        "Tense": "Imp",
        "VerbForm": "Fin",
    },
    "Vmii1-----y": {
        "Mood": "Ind",
        POS: VERB,
        "Person": "one",
        "Tense": "Imp",
        
        "VerbForm": "Fin",
    },
    "Vmii2p": {
        "Mood": "Ind",
        "Number": "Plur",
        POS: VERB,
        "Person": "two",
        "Tense": "Imp",
        "VerbForm": "Fin",
    },
    "Vmii2s": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: VERB,
        "Person": "two",
        "Tense": "Imp",
        "VerbForm": "Fin",
    },
    "Vmii3p": {
        "Mood": "Ind",
        "Number": "Plur",
        POS: VERB,
        "Person": "three",
        "Tense": "Imp",
        "VerbForm": "Fin",
    },
    "Vmii3p----y": {
        "Mood": "Ind",
        "Number": "Plur",
        POS: VERB,
        "Person": "three",
        "Tense": "Imp",
        
        "VerbForm": "Fin",
    },
    "Vmii3s": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: VERB,
        "Person": "three",
        "Tense": "Imp",
        "VerbForm": "Fin",
    },
    "Vmil3p": {
        "Mood": "Ind",
        "Number": "Plur",
        POS: VERB,
        "Person": "three",
        
        "VerbForm": "Fin",
    },
    "Vmil3s": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: VERB,
        "Person": "three",
        
        "VerbForm": "Fin",
    },
    "Vmip1p": {
        "Mood": "Ind",
        "Number": "Plur",
        POS: VERB,
        "Person": "one",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "Vmip1s": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: VERB,
        "Person": "one",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "Vmip1s----y": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: VERB,
        "Person": "one",
        "Tense": "Pres",
        
        "VerbForm": "Fin",
    },
    "Vmip2p": {
        "Mood": "Ind",
        "Number": "Plur",
        POS: VERB,
        "Person": "two",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "Vmip2s": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: VERB,
        "Person": "two",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "Vmip3": {
        "Mood": "Ind",
        POS: VERB,
        "Person": "three",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "Vmip3-----y": {
        "Mood": "Ind",
        POS: VERB,
        "Person": "three",
        "Tense": "Pres",
        
        "VerbForm": "Fin",
    },
    "Vmip3p": {
        "Mood": "Ind",
        "Number": "Plur",
        POS: AUX,
        "Person": "three",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "Vmip3s": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: VERB,
        "Person": "three",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "Vmip3s----y": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: AUX,
        "Person": "three",
        "Tense": "Pres",
        
        "VerbForm": "Fin",
    },
    "Vmis1p": {
        "Mood": "Ind",
        "Number": "Plur",
        POS: VERB,
        "Person": "one",
        "Tense": "Past",
        "VerbForm": "Fin",
    },
    "Vmis1s": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: VERB,
        "Person": "one",
        "Tense": "Past",
        "VerbForm": "Fin",
    },
    "Vmis3p": {
        "Mood": "Ind",
        "Number": "Plur",
        POS: VERB,
        "Person": "three",
        "Tense": "Past",
        "VerbForm": "Fin",
    },
    "Vmis3s": {
        "Mood": "Ind",
        "Number": "Sing",
        POS: VERB,
        "Person": "three",
        "Tense": "Past",
        "VerbForm": "Fin",
    },
    "Vmm-2p": {
        "Mood": "Imp",
        "Number": "Plur",
        POS: VERB,
        "Person": "two",
        "VerbForm": "Fin",
    },
    "Vmm-2s": {
        "Mood": "Imp",
        "Number": "Sing",
        POS: VERB,
        "Person": "two",
        "VerbForm": "Fin",
    },
    "Vmnp": {POS: VERB, "Tense": "Pres", "VerbForm": "Inf"},
    "Vmp--pf": {"Gender": "Fem", "Number": "Plur", POS: VERB, "VerbForm": "Part"},
    "Vmp--pm": {"Gender": "Masc", "Number": "Plur", POS: VERB, "VerbForm": "Part"},
    "Vmp--sf": {"Gender": "Fem", "Number": "Sing", POS: VERB, "VerbForm": "Part"},
    "Vmp--sm": {"Gender": "Masc", "Number": "Sing", POS: VERB, "VerbForm": "Part"},
    "Vmsp3": {
        "Mood": "Sub",
        POS: VERB,
        "Person": "three",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "Vmsp3-----y": {
        "Mood": "Sub",
        POS: VERB,
        "Person": "three",
        "Tense": "Pres",
        
        "VerbForm": "Fin",
    },
    "X": {POS: X},
    "Y": {"Abbr": "Yes", POS: X},
    "Yn": {"Abbr": "Yes", POS: NOUN},
    "Ynmsry": {
        "Abbr": "Yes",
        
        "Gender": "Masc",
        "Number": "Sing",
        POS: NOUN,
    },
}



# coding: utf-8
from __future__ import unicode_literals

import pytest
from spacy.kb import KnowledgeBase
from spacy.util import ensure_path
from spacy.lang.en import English

from ..util import make_tempdir


def test_issue4674():
    """Test that setting entities with overlapping identifiers does not mess up IO"""
    nlp = English()
    kb = KnowledgeBase(nlp.vocab, entity_vector_length=3)

    vector1 = [0.9, 1.1, 1.01]
    vector2 = [1.8, 2.25, 2.01]
    with pytest.warns(UserWarning):
        kb.set_entities(
            entity_list=["Q1", "Q1"],
            freq_list=[32, 111],
            vector_list=[vector1, vector2],
        )

    assert kb.get_size_entities() == 1

    # dumping to file & loading back in
    with make_tempdir() as d:
        dir_path = ensure_path(d)
        if not dir_path.exists():
            dir_path.mkdir()
        file_path = dir_path / "kb"
        kb.dump(str(file_path))

        kb2 = KnowledgeBase(vocab=nlp.vocab, entity_vector_length=3)
        kb2.load_bulk(str(file_path))

    assert kb2.get_size_entities() == 1



# coding: utf-8
from __future__ import unicode_literals


def test_sv_tokenizer_handles_long_text(sv_tokenizer):
    text = """Det var så härligt ute på landet. Det var sommar, majsen var gul, havren grön,
höet var uppställt i stackar nere vid den gröna ängen, och där gick storken på sina långa,
röda ben och snackade engelska, för det språket hade han lärt sig av sin mor.

Runt om åkrar och äng låg den stora skogen, och mitt i skogen fanns djupa sjöar; jo, det var verkligen trevligt ute på landet!"""
    tokens = sv_tokenizer(text)
    assert len(tokens) == 86


def test_sv_tokenizer_handles_trailing_dot_for_i_in_sentence(sv_tokenizer):
    text = "Provar att tokenisera en mening med ord i."
    tokens = sv_tokenizer(text)
    assert len(tokens) == 9



# coding: utf8
from __future__ import unicode_literals


# Stop words
# Link to greek stop words: https://www.translatum.gr/forum/index.php?topic=3550.0?topic=3550.0
STOP_WORDS = set(
    """
αδιάκοπα αι ακόμα ακόμη ακριβώς άλλα αλλά αλλαχού άλλες άλλη άλλην
άλλης αλλιώς αλλιώτικα άλλο άλλοι αλλοιώς αλλοιώτικα άλλον άλλος άλλοτε αλλού
άλλους άλλων άμα άμεσα αμέσως αν ανά ανάμεσα αναμεταξύ άνευ αντί αντίπερα αντίς
άνω ανωτέρω άξαφνα απ απέναντι από απόψε άρα άραγε αρκετά αρκετές
αρχικά ας αύριο αυτά αυτές αυτή αυτήν αυτής αυτό αυτοί αυτόν αυτός αυτού αυτούς
αυτών αφότου αφού

βέβαια βεβαιότατα

γι για γιατί γρήγορα γύρω

δα δε δείνα δεν δεξιά δήθεν δηλαδή δι δια διαρκώς δικά δικό δικοί δικός δικού
δικούς διόλου δίπλα δίχως

εάν εαυτό εαυτόν εαυτού εαυτούς εαυτών έγκαιρα εγκαίρως εγώ εδώ ειδεμή είθε είμαι
είμαστε είναι εις είσαι είσαστε είστε είτε είχα είχαμε είχαν είχατε είχε είχες έκαστα
έκαστες έκαστη έκαστην έκαστης έκαστο έκαστοι έκαστον έκαστος εκάστου εκάστους εκάστων
εκεί εκείνα εκείνες εκείνη εκείνην εκείνης εκείνο εκείνοι εκείνον εκείνος εκείνου
εκείνους εκείνων εκτός εμάς εμείς εμένα εμπρός εν ένα έναν ένας ενός εντελώς εντός
εναντίον  εξής  εξαιτίας  επιπλέον επόμενη εντωμεταξύ ενώ εξ έξαφνα εξήσ εξίσου έξω επάνω
επειδή έπειτα επί επίσης επομένως εσάς εσείς εσένα έστω εσύ ετέρα ετέραι ετέρας έτερες
έτερη έτερης έτερο έτεροι έτερον έτερος ετέρου έτερους ετέρων ετούτα ετούτες ετούτη ετούτην
ετούτης ετούτο ετούτοι ετούτον ετούτος ετούτου ετούτους ετούτων έτσι εύγε ευθύς ευτυχώς εφεξής
έχει έχεις έχετε έχομε έχουμε έχουν εχτές έχω έως έγιναν  έγινε  έκανε  έξι  έχοντας

η ήδη ήμασταν ήμαστε ήμουν ήσασταν ήσαστε ήσουν ήταν ήτανε ήτοι ήττον

θα

ι ιδία ίδια ίδιαν ιδίας ίδιες ίδιο ίδιοι ίδιον ίδιοσ ίδιος ιδίου ίδιους ίδιων ιδίως ιι ιιι
ίσαμε ίσια ίσως

κάθε καθεμία καθεμίας καθένα καθένας καθενός καθετί καθόλου καθώς και κακά κακώς καλά
καλώς καμία καμίαν καμίας κάμποσα κάμποσες κάμποση κάμποσην κάμποσης κάμποσο κάμποσοι
κάμποσον κάμποσος κάμποσου κάμποσους κάμποσων κανείς κάνεν κανένα κανέναν κανένας
κανενός κάποια κάποιαν κάποιας κάποιες κάποιο κάποιοι κάποιον κάποιος κάποιου κάποιους
κάποιων κάποτε κάπου κάπως κατ κατά κάτι κατιτί κατόπιν κάτω κιόλας κλπ κοντά κτλ κυρίως

λιγάκι λίγο λιγότερο λόγω λοιπά λοιπόν

μα μαζί μακάρι μακρυά μάλιστα μάλλον μας με μεθαύριο μείον μέλει μέλλεται μεμιάς μεν
μερικά μερικές μερικοί μερικούς μερικών μέσα μετ μετά μεταξύ μέχρι μη μήδε μην μήπως
μήτε μια μιαν μιας μόλις μολονότι μονάχα μόνες μόνη μόνην μόνης μόνο μόνοι μονομιάς
μόνος μόνου μόνους μόνων μου μπορεί μπορούν μπρος μέσω  μία  μεσώ

να ναι νωρίς

ξανά ξαφνικά

ο οι όλα όλες όλη όλην όλης όλο ολόγυρα όλοι όλον ολονέν όλος ολότελα όλου όλους όλων
όλως ολωσδιόλου όμως όποια οποιαδήποτε οποίαν οποιανδήποτε οποίας οποίος οποιασδήποτε οποιδήποτε
όποιες οποιεσδήποτε όποιο οποιοδηήποτε όποιοι όποιον οποιονδήποτε όποιος οποιοσδήποτε
οποίου οποιουδήποτε οποίους οποιουσδήποτε οποίων οποιωνδήποτε όποτε οποτεδήποτε όπου
οπουδήποτε όπως ορισμένα ορισμένες ορισμένων ορισμένως όσα οσαδήποτε όσες οσεσδήποτε
όση οσηδήποτε όσην οσηνδήποτε όσης οσησδήποτε όσο οσοδήποτε όσοι οσοιδήποτε όσον οσονδήποτε
όσος οσοσδήποτε όσου οσουδήποτε όσους οσουσδήποτε όσων οσωνδήποτε όταν ότι οτιδήποτε
ότου ου ουδέ ούτε όχι οποία  οποίες  οποίο  οποίοι  οπότε  ος

πάνω  παρά  περί  πολλά  πολλές  πολλοί  πολλούς  που  πρώτα  πρώτες  πρώτη  πρώτο  πρώτος  πως
πάλι πάντα πάντοτε παντού πάντως πάρα πέρα πέρι περίπου περισσότερο πέρσι πέρυσι πια πιθανόν
πιο πίσω πλάι πλέον πλην ποιά ποιάν ποιάς ποιές ποιό ποιοί ποιόν ποιός ποιού ποιούς
ποιών πολύ πόσες πόση πόσην πόσης πόσοι πόσος πόσους πότε ποτέ πού πούθε πουθενά πρέπει
πριν προ προκειμένου πρόκειται πρόπερσι προς προτού προχθές προχτές πρωτύτερα πώς

σαν σας σε σεις σου στα στη στην στης στις στο στον στου στους στων συγχρόνως
συν συνάμα συνεπώς συχνάς συχνές συχνή συχνήν συχνής συχνό συχνοί συχνόν
συχνός συχνού συχνούς συχνών συχνώς σχεδόν

τα τάδε ταύτα ταύτες ταύτη ταύτην ταύτης ταύτοταύτον ταύτος ταύτου ταύτων τάχα τάχατε
τελευταία  τελευταίο  τελευταίος  τού  τρία  τρίτη  τρεις τελικά τελικώς τες τέτοια τέτοιαν
τέτοιας τέτοιες τέτοιο τέτοιοι τέτοιον τέτοιος τέτοιου
τέτοιους τέτοιων τη την της τι τίποτα τίποτε τις το τοι τον τοσ τόσα τόσες τόση τόσην
τόσης τόσο τόσοι τόσον τόσος τόσου τόσους τόσων τότε του τουλάχιστο τουλάχιστον τους τούς τούτα
τούτες τούτη τούτην τούτης τούτο τούτοι τούτοις τούτον τούτος τούτου τούτους τούτων τυχόν
των τώρα

υπ υπέρ υπό υπόψη υπόψιν ύστερα

χωρίς χωριστά

ω ως ωσάν ωσότου ώσπου ώστε ωστόσο ωχ
""".split()
)



# coding: utf8
from __future__ import unicode_literals


"""
Example sentences to test spaCy and its language models.

>>> from spacy.lang.en.examples import sentences
>>> docs = nlp.pipe(sentences)
"""


sentences = [
    "Apple is looking at buying U.K. startup for $1 billion",
    "Autonomous cars shift insurance liability toward manufacturers",
    "San Francisco considers banning sidewalk delivery robots",
    "London is a big city in the United Kingdom.",
    "Where are you?",
    "Who is the president of France?",
    "What is the capital of the United States?",
    "When was Barack Obama born?",
]



# encoding: utf8
from __future__ import unicode_literals

from ...symbols import POS, PUNCT, INTJ, X, SYM, ADJ, AUX, ADP, CONJ, NOUN, PRON
from ...symbols import VERB, ADV, PROPN, NUM, DET

# 은전한닢(mecab-ko-dic)의 품사 태그를 universal pos tag로 대응시킴
# https://docs.google.com/spreadsheets/d/1-9blXKjtjeKZqsf4NzHeYJCrr49-nXeRF6D80udfcwY/edit#gid=589544265
# https://universaldependencies.org/u/pos/
TAG_MAP = {
    # J.{1,2} 조사
    "JKS": {POS: ADP},
    "JKC": {POS: ADP},
    "JKG": {POS: ADP},
    "JKO": {POS: ADP},
    "JKB": {POS: ADP},
    "JKV": {POS: ADP},
    "JKQ": {POS: ADP},
    "JX": {POS: ADP},  # 보조사
    "JC": {POS: CONJ},  # 접속 조사
    "MAJ": {POS: CONJ},  # 접속 부사
    "MAG": {POS: ADV},  # 일반 부사
    "MM": {POS: DET},  # 관형사
    "XPN": {POS: X},  # 접두사
    # XS. 접미사
    "XSN": {POS: X},
    "XSV": {POS: X},
    "XSA": {POS: X},
    "XR": {POS: X},  # 어근
    # E.{1,2} 어미
    "EP": {POS: X},
    "EF": {POS: X},
    "EC": {POS: X},
    "ETN": {POS: X},
    "ETM": {POS: X},
    "IC": {POS: INTJ},  # 감탄사
    "VV": {POS: VERB},  # 동사
    "VA": {POS: ADJ},  # 형용사
    "VX": {POS: AUX},  # 보조 용언
    "VCP": {POS: ADP},  # 긍정 지정사(이다)
    "VCN": {POS: ADJ},  # 부정 지정사(아니다)
    "NNG": {POS: NOUN},  # 일반 명사(general noun)
    "NNB": {POS: NOUN},  # 의존 명사
    "NNBC": {POS: NOUN},  # 의존 명사(단위: unit)
    "NNP": {POS: PROPN},  # 고유 명사(proper noun)
    "NP": {POS: PRON},  # 대명사
    "NR": {POS: NUM},  # 수사(numerals)
    "SN": {POS: NUM},  # 숫자
    # S.{1,2} 부호
    # 문장 부호
    "SF": {POS: PUNCT},  # period or other EOS marker
    "SE": {POS: PUNCT},
    "SC": {POS: PUNCT},  # comma, etc.
    "SSO": {POS: PUNCT},  # open bracket
    "SSC": {POS: PUNCT},  # close bracket
    "SY": {POS: SYM},  # 기타 기호
    "SL": {POS: X},  # 외국어
    "SH": {POS: X},  # 한자
}
